#ifndef APPLICATION_H_INCLUDED
#define APPLICATION_H_INCLUDED

// A general application.
// This could solve several problems:
// - the help text could be automatically generated
// - duplicate main code could be removed
// - it would be easy to parse options
// - all applications could be linked to a single file thereby saving diskspace

class Application{
  // Nested Option Classes
  class Option{
  protected:
    static Option *constructionList;
  public:
    Option();
    virtual bool matches(const char *s)=0;
    virtual void parseValue(int t, char **argv, bool *ok, int *numberOfArgumentsParsed)=0;
    static Option *getOptionList();
    Option *next;
  };
 protected:
  class StringMatchingOption: public Option
    {
    protected:
      const char *matchString;
      bool isExactMatch(const char *s);
    public:
      StringMatchingOption(const char *s);
      bool matches(const char *s);
    };

  class SimpleOption: public StringMatchingOption{
    bool value;
  public:
    SimpleOption(const char *s);
    void parseValue(int t, char **argv, bool *ok, int *numberOfArgumentsParsed);
    bool getValue();
  };

  class ValueOption: public StringMatchingOption{
  public:
    virtual void assignValue(const char *s)=0;
    ValueOption(const char *s);
    void parseValue(int t, char **argv, bool *ok, int *numberOfArgumentsParsed);
  };

  class StringOption: public ValueOption{
    const char *value;
  public:
    StringOption(const char *s, const char *initialValue=0);
    void assignValue(const char *s);
    const char *getValue();
  };  

  class IntegerOption: public ValueOption{
    int value;
    bool hasRange;
    int lower,upper;
  public:
    IntegerOption(const char *s, int initialValue=0);
    IntegerOption(const char *s, int initialValue, int lower, int upper);
    void assignValue(const char *s);
    int getValue();
  };

  class ZeroOneOption: public IntegerOption{
  public:
    ZeroOneOption(const char *s, int initialValue=0);
  };

  // Application members
 private:
  static class Application *applicationList;
  class Application::Option* optionList;
 public:
  Application();
  class Application *next;
  bool parseOptions(int argc, char **argv);
  void registerOptions();
  virtual void printHelp();
  virtual int main()=0;
  virtual const char *helpText()=0;
  friend int main(int argc, char *argv[]);
};


class CatsApplication : public Application
{
  void printHelp();
};


#endif
