#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <vector>
#include <list>

using namespace std;


#include "vektor.h"
#include "lll.h"
#include "print.h"
#include "division.h"
#include "buchberger.h"
#include "toric.h"
#include "flip.h"
#include "lp.h"
#include "parser.h"
#include "radical.h"
#include "printer.h"
#include "macaulay2.h"
#include "decompose.h"

#include "ep_latex.h"
#include "ep_tigers.h"

#include "ea_exhaustive.h"
#include "ea_reverse.h"
#include "ea_exhaustiveradical.h"
#include "ea_reverseradical.h"
#include "ea_revlex.h"

#include "result.h"

#include "testresult.h"
#include "normaliz.h"
#include "xml.h"
#include "counter.h"

#include "application.h"

Macaulay2Pipe M2;





















int computeKerDim(basis_i B)
{
  basis_i M=mlll(B);
  int kerdim=0;
  while((kerdim<B.size())&&(B[kerdim].iszero()))kerdim++;
  return kerdim;
}



/*static void printhelp(FILE *f=stdout)
{
  char *help="";

  fprintf(f,help);

  if(lp_soplexlinked)
    fprintf(f,"SoPlex library is available.\n");
  else
    fprintf(f,"SoPlex library is not available.\n");
}
*/

class MyTarget:public EnumerationTarget
{
 public:
  int n;
  virtual void beginEnumeration(const gbasis &g)
    {
      n=0;
    }
  virtual void endEnumeration()
    {
    }
  virtual void beginRadical(const gbasis &radical)
    {
    }
  virtual void endRadical()
    {
    }
  virtual bool basis(const gbasis &g)
    {
      n++;

      return true;
    }
};

void runExample(vektor_i v, FILE *fxml)
{
  int d=v.size();
  gbasis a;

  for(int i=0;i<d;i++)
    {
      if(i==d-1)a.push_back(v);
      else
	{
	  vektor_i u(d);
	  for(int j=0;j<d;j++)u[j]=j==i;
	  a.push_back(u);
	}
    }
  
  gbasis a2=normaliz(a);

  {
    gbasis::iterator j=a2.begin();
    for(gbasis::iterator i=a.begin();i!=a.end();i++)
      {
	assert((*i-*j).iszero());
	j++;
      }
  }

  print(a2);



  basis_i B(a2.size());

  {
    int t=0;
    for(gbasis::iterator i=a2.begin();i!=a2.end();i++)
      B[t++]=*i;
  }

  writeExampleToXml(B,fxml);
  //B=tobasis_i("{(1,1)(1,2)(1,3)(1,4)}");//to test M2.isHomogeneous

  //print(B,stderr);

  gbasis g=toric1(B);


  printf("Changing term order to rev lex with $a<b<c....$\n\n");

  vektor_i::setcheapest2(0,a2.size());
  //  vektor_i::setcheapest(a2.size()-1);
  vektor_i::TERMORDER=REVERSELEXICOGRAPHIC;
  buchberger(g);
  gbasisminimal(&g);
  gbasisautoreduction(&g);


  printt(g);

  gbasis r=initialradical(g);

  printf("radical:\n\n");
  printt(r);

  M2.setPolynomialRing(g);
  if(M2.isHomogeneousIdeal(g))
    fprintf(stderr,"true");
  else
    fprintf(stderr,"false");


  /*
  //ExhaustiveSearchFixedRadical ea;
  //ReverseSearchFixedRadical ea;
  ReverseSearch ea;
  MyTarget myTarget;

  ea.setEnumerationTarget(&myTarget);
  ea.enumerate(g);
  printf("Number of initial ideals supported on the coarse triangulation:%i\n\n",myTarget.n);
  */
}

bool stringSearch(FILE *f, char *s)
{
  // no duplicates in s!!!!!
  int pos=0;
  int l=strlen(s);

  while(!feof(f) && pos<l)
    {
      int c=getc(f);

      if(c==s[pos])
	pos++;
      else
	{
	  if(pos)ungetc(c,f);
	  pos=0;
	}
    }
  return pos==l;
}


void runExamples(char *s)
{
  FILE *fxml =fopen("FZexamples.xml","w");

  int n=0;

  FILE *f;
  f=fopen(s,"r");
  assert(f);
  while(stringSearch(f,"CONE")&& n++<3)//370
    {
      char temp[1024];
      fgets(temp,1024,f);
      int l=strlen(temp);
      for(int i=0;i<l-1;i++)
	{
	  if(temp[i]==' ' || temp[i]=='\t')
	    {
	      if(temp[i-1]>='0' && temp[i-1]<='9')temp[i]=',';
	    }
	  if(temp[i]==')')temp[i+1]=0;
	}
      //      vektor_i v(temp);

      vektor_i v=StringParser(temp).parseIntegerVector();//24. september

      runExample(v,fxml);
      //      print(v);
    }
  fclose(fxml);
}




//--------------------------------------------------
// Degree conjecture test
//--------------------------------------------------
//(D): Does the toric ideal I_A of a normal graded A have an initial ideal 
//     of degree at most the rank of A ?

class MyDegreeConjectureTarget:public EnumerationTarget
{
 public:
  gbasis why;
  int cc;
  bool counterExampleFound;
  int n;
  virtual void beginEnumeration(const gbasis &g)
    {
      n=0;
      counterExampleFound=true;
    }
  virtual void endEnumeration()
    {
    }
  virtual void beginRadical(const gbasis &radical, int numberOfSymmetries)
    {
    }
  virtual void endRadical()
    {
    }
  virtual bool basis(const gbasis &g)
    {
      int rankA=g.begin()->size()-cc;
      n++;

      gbasis initial=initialIdeal(g);

      if(degree(g)<=rankA)
	{
	  why=g;
	  counterExampleFound=false;
	  return false;
	}
      return true;
    }
};

void testDegreeConjecture(gbasis &g, int cc)
{
  ReverseSearch ea;
  MyDegreeConjectureTarget target;

  target.cc=cc;
  ea.setEnumerationTarget(&target);
  ea.enumerate(g);
  if(target.counterExampleFound)
    {
      fprintf(stderr,"Counter Example Found !!\n");

      AsciiPrinter(stderr).printGroebnerBasis(g);
      fprintf(stderr,"\n");
      fprintf(stderr,"Counter Example Found !!\n");
      exit(1);
    }
  else
    {
      fprintf(stderr,"This was not a counter example, search terminated in %i steps\n",target.n);
      fprintf(stderr,"The following reduced Gr\"obner basis has degree less than or equal to the rank of A:\n");
      AsciiPrinter(stderr).printGroebnerBasis(target.why);
      fprintf(stderr,"\n");
    }
}

//--------------------------------------------------
// E=>D conjecture test
//--------------------------------------------------
//(E=>D): Conjecture: All initial ideals of I_A when A is normal & graded 
//       that does not have embedded primes has degree at most d = rank(A).

class MyEImpliesDConjectureTarget:public EnumerationTarget
{
 public:
  gbasis why;
  int cc;
  bool counterExampleFound;
  int n;
  virtual void beginEnumeration(const gbasis &g)
    {
      n=0;
      counterExampleFound=false;
    }
  virtual void endEnumeration()
    {
    }
  virtual void beginRadical(const gbasis &radical, int numberOfSymmetries)
    {
    }
  virtual void endRadical()
    {
    }
  virtual bool basis(const gbasis &g)
    {
      int rankA=g.begin()->size()-cc;
      //      fprintf(stderr,"rankA:%i\n",rankA);
      n++;

      gbasis initial=initialIdeal(g);

      if(degree(g)>rankA)
	{
	  //	  fprintf(stderr,"We have to test for embedded primes\n");
	  if(!hasEmbeddedPrimes(initial))
	    {
	      why=g;
	      counterExampleFound=true;
	      return false;
	    }
	}
      return true;
    }
};


void testEImpliesDConjecture(gbasis &g, int cc)
{
  ReverseSearch ea;
  MyEImpliesDConjectureTarget target;

  target.cc=cc;
  ea.setEnumerationTarget(&target);
  ea.enumerate(g);
  if(target.counterExampleFound)
    {
      fprintf(stderr,"Counter Example Found !!\n");

      fprintf(stderr,"The following initial ideal has no embedded primes and has degree greater than the rank of A:\n");
      AsciiPrinter(stderr).printGroebnerBasis(target.why);
      fprintf(stderr,"\nThe standard pairs are:\n");
      AsciiPrinter(stderr).printStandardPairList(computeStandardPairs(initialIdeal(target.why)));

      fprintf(stderr,"Counter Example Found !!\n");
      exit(1);
    }
  else
    fprintf(stderr,"This was not a counter example, search terminated in %i steps\n",target.n);
    fprintf(stderr,"All initial ideals without embedded primes have degree less than or equal to the rank of A\n",target.n);
}

//--------------------------------------------------
// Embedded prime conjecture test
//--------------------------------------------------

class MyEmbeddedPrimeTarget:public EnumerationTarget
{
 public:
  int cc;
  bool counterExampleFound;
  gbasis why;
  int n;
  virtual void beginEnumeration(const gbasis &g)
    {
      n=0;
      counterExampleFound=true;
    }
  virtual void endEnumeration()
    {
    }
  virtual void beginRadical(const gbasis &radical, int numberOfSymmetries)
    {
    }
  virtual void endRadical()
    {
    }
  virtual bool basis(const gbasis &g)
    {
      n++;

      gbasis initial=initialIdeal(g);

      if(n==2000)Timer::printList();

      if(!hasEmbeddedPrimes(initial))
	{
	  why=g;
	  counterExampleFound=false;
	  return false;
	}
      return true;
    }
};


void testEmbeddedPrimeConjecture(gbasis &g, int cc)
{
  ReverseSearch ea;
  MyEmbeddedPrimeTarget target;

  target.cc=cc;
  ea.setEnumerationTarget(&target);
  ea.enumerate(g);
  if(target.counterExampleFound)
    {
      fprintf(stderr,"Counter Example Found !!\n");
      AsciiPrinter(stderr).printGroebnerBasis(g);
      fprintf(stderr,"\n");
      fprintf(stderr,"Counter Example Found !!\n");
      exit(1);
    }
  else
    {
      fprintf(stderr,"This was not a counter example, search terminated in %i steps\n",target.n);
      fprintf(stderr,"The following reduced Gr\"obner basis has an initial ideal without embedded primes:\n");
      AsciiPrinter(stderr).printGroebnerBasis(target.why);
      fprintf(stderr,"\nThe standard pairs are:\n");
      AsciiPrinter(stderr).printStandardPairList(computeStandardPairs(initialIdeal(target.why)));
    }
}


//--------------------------------------------------
// Cohen-Macaulay conjecture test
//--------------------------------------------------

class MyCohenMacaulayTarget:public EnumerationTarget
{
 public:
  int cc;
  bool counterExampleFound;
  bool counting;
  gbasis why;
  int n;
  int numberOfIdealsWithoutEmbedded;
  int numberOfCohenMacaulayIdeals;
  virtual void beginEnumeration(const gbasis &g)
    {
      n=0;
      numberOfIdealsWithoutEmbedded=0;
      numberOfCohenMacaulayIdeals=0;
      counterExampleFound=true;
    }
  virtual void endEnumeration()
    {
    }
  virtual void beginRadical(const gbasis &radical, int numberOfSymmetries)
    {
    }
  virtual void endRadical()
    {
    }
  virtual bool basis(const gbasis &g)
    {
      n++;

      gbasis initial=initialIdeal(g);

      if(n==2000)Timer::printList();

      if(!hasEmbeddedPrimes(initial))
	{
	  numberOfIdealsWithoutEmbedded++;
	  fprintf(stderr,"We have to test for Cohen-Macaulayness\n");
	  int pDim=M2.getPdimCokerGensMonomial(initial);
	  fprintf(stderr,"CM test: pdim coker gens:%i == %i\n",pDim,cc);
	  //	  fprintf(stderr,"Hmmm... unfortunately I forgot how to do that but M2 says pdim coker gens is %i\n",pDim);
	  if(pDim==cc)
	    {
	      numberOfCohenMacaulayIdeals++;
	      if(!counting)
		{
		  why=g;
		  counterExampleFound=false;
		  return false;
		}
	    }
	}
      return true;
    }
};


void testCohenMacaulayConjecture(gbasis &g, int cc, bool count)
{
  M2.setPolynomialRing(g.begin()->size());
  ReverseSearch ea;
  MyCohenMacaulayTarget target;

  target.cc=cc;
  target.counting=count;
  ea.setEnumerationTarget(&target);
  ea.enumerate(g);

  if(count)
    {
      printf("Number of initial ideals:%i\n",target.n);
      printf("Number of initial ideals without embedded primes:%i\n",target.numberOfIdealsWithoutEmbedded);
      printf("Number of initial ideals that are Cohen Macaulay:%i\n",target.numberOfCohenMacaulayIdeals);
    }
  else
    {
      if(target.counterExampleFound)
	{
	  fprintf(stderr,"Counter Example Found !!\n");
	  AsciiPrinter(stderr).printGroebnerBasis(g);
	  fprintf(stderr,"\n");
	  fprintf(stderr,"Counter Example Found !!\n");
	  exit(1);
	}
      else
	{
	  fprintf(stderr,"This was not a counter example, search terminated in %i steps\n",target.n);
	  fprintf(stderr,"The following reduced Gr\"obner basis has an initial ideal which is Cohen Macaulay:\n");
	  AsciiPrinter(stderr).printGroebnerBasis(target.why);
	  fprintf(stderr,"\n");
	}
    }
}


//--------------------------------------------------
// Reverse Lexicographic Degree conjecture test
//--------------------------------------------------
//(RLD): Are all initial ideals of a toric ideal I_A of a normal graded A
//       generated in degree at most the rank of A ?

class MyRLDConjectureTarget:public EnumerationTarget
{
 public:
  set<int> hasDegreeLargerThanRank;
  set<int> hasDegreeAtMostRank;
  gbasis why;
  int cc;
  bool counterExampleFound;
  int n;
  virtual void beginEnumeration(const gbasis &g)
    {
      n=0;
      counterExampleFound=false;
    }
  virtual void endEnumeration()
    {
    }
  virtual void beginRadical(const gbasis &radical, int numberOfSymmetries)
    {
    }
  virtual void endRadical()
    {
    }
  virtual bool basis(const gbasis &g)
    {
      int rankA=g.begin()->size()-cc;
      n++;

      gbasis initial=initialIdeal(g);

      if(degree(g)>rankA)
	{
	  hasDegreeLargerThanRank.insert(*ReverseLexicographicEnumeration::permutation.begin());
	  why=g;
	  counterExampleFound=true;
	  //	  return false;
	}
      else
	{
	  hasDegreeAtMostRank.insert(*ReverseLexicographicEnumeration::permutation.begin());
	}
      return true;
    }
};

void testRLDConjecture(gbasis &g, int cc)
{
  ReverseLexicographicEnumeration ea;
  MyRLDConjectureTarget target;

  target.cc=cc;
  ea.setEnumerationTarget(&target);

  ea.enumerate(g);

  fprintf(stderr,"Printing the minimal variables, which violate the conjecture:\n");
  for(set<int>::const_iterator i=target.hasDegreeLargerThanRank.begin();i!=target.hasDegreeLargerThanRank.end();i++)
    fprintf(stderr,"%c ",(*i)+'a');
  fprintf(stderr,"\nPrinting the minimal variables, which does not violate the conjecture:\n");
  for(set<int>::const_iterator i=target.hasDegreeAtMostRank.begin();i!=target.hasDegreeAtMostRank.end();i++)
    fprintf(stderr,"%c ",(*i)+'a');
  fprintf(stderr,"\n");

  if(target.counterExampleFound)
    {
      fprintf(stderr,"Counter Example Found !!\n");
      AsciiPrinter(stderr).printGroebnerBasis(target.why);
      fprintf(stderr,"\n");
      fprintf(stderr,"Counter Example Found !!\n");
      exit(1);
    }
  else
    {
      fprintf(stderr,"This was not a counter example, search terminated in %i steps\n",target.n);
    }
}

int myRand(int modulo)
{
  return (unsigned int)rand()%modulo;
}


void runRandomTests()
{
  int r=6;
  int d=4;
  int u,n,s;

  int counter=0;

  //  fprintf(stderr,"Type in dimension (d):");
  //  scanf("%i",&d);
  d=3;
  fprintf(stderr,"Type in number of random vector (r):");
  scanf("%i",&r);
  fprintf(stderr,"Type in upper limit of the range of the entries (u):");
  scanf("%i",&u);
  fprintf(stderr,"Type in maximal allowable number of variables (n):");
  scanf("%i",&n);
  fprintf(stderr,"Type in random seed (number of rand()'s to skip):");
  scanf("%i",&s);

  while(s-->0)rand(); //we really should find a better way to initialize rand

  while(1)
    {
      gbasis a;

      for(int i=0;i<r;i++)
	{
	  vektor_i v(d);
	  for(int j=0;j<d;j++)v[j]=myRand(u+1);
	  
	  v[d-1]=1;//homogeneous
	  a.push_back(v);
	}
      
      gbasis a2=normaliz(a);

      //            a2=a;
      counter++;
      fprintf(stderr,"NumberOfTimes normaliz has been called: %i. Number of columns in A: %i\n",counter,a2.size());

      basis_i B(a2.size());
      
      {
	int t=0;
	for(gbasis::iterator i=a2.begin();i!=a2.end();i++)
	  B[t++]=*i;
      }
      
     
      if(B.size()<=n)
	{

	  fprintf(stderr,"--------------------------------------------------------\n");
	  print(B,stderr);
	  fprintf(stderr,"--------------------------------------------------------\n");
	  
	  gbasis g=toric1(B);

	  if(g.size()>0)
	    testEImpliesDConjecture(g,computeKerDim(B));
	  else
	    fprintf(stderr,"Trivial ideal\n");
	}
    }
}



void testConjectures(basis_i B, bool E, bool CM, bool D, bool EImpliesD, bool RLD, bool count)
{
  {
    FILE *f=fopen("singleExample.xml","w");
    writeExampleToXml(B,f);
    fclose(f);
  }

  AsciiPrinter(stderr).printMatrix(B);

  gbasis g=toric1(B);

  int kerdim=computeKerDim(B);
  
  if(g.size()>0)
    {
      vektor_i::setcheapest2(0,B.size());
      //      vektor_i::setcheapest2(0,a2.size());
      //  vektor_i::setcheapest(a2.size()-1);
      vektor_i::TERMORDER=REVERSELEXICOGRAPHIC;
      buchberger(g);
      gbasisminimal(&g);
      gbasisautoreduction(&g);
      
      if(E)
	{
	  fprintf(stderr,"Testing (E) conjecture.\n");
	  testEmbeddedPrimeConjecture(g,kerdim);
	}
      if(CM)
	{
	  fprintf(stderr,"Testing (CM) conjecture.\n");
	  testCohenMacaulayConjecture(g,kerdim,count);
	}
      if(isGraded(g))
	{
	  if(RLD)
	    {
	      fprintf(stderr,"test\n");
	      printConvexHullInfo(B);
	      fprintf(stderr,"Testing (RLD) conjecture.\n");
	      testRLDConjecture(g,kerdim);
	    }
	  if(D)
	    {
	      fprintf(stderr,"Testing (D) conjecture.\n");
	      testDegreeConjecture(g,kerdim);
	    }
	  if(EImpliesD)
	    {
	      fprintf(stderr,"Testing (E=>D) conjecture.\n");
	      testEImpliesDConjecture(g,kerdim);
	    }
	}
      else
	{
	  fprintf(stderr,"The toric ideal is not graded so (D) and (E) conjectures were not tested\n");
	}
    }
  else
    fprintf(stderr,"Trivial ideal\n");

  printf("done\n");
}

class ConjectureApplication : public CatsApplication
{
  const char *helpText()
  {
    return "This is a program for testing some conjectures on toric ideals.\n"
      "The conjectures are:\n"
      "  E\n"
      "  CM\n"
      "  D\n"
      "  E=>D\n"
      "The input to the program is a matrix in CaTS' format.\n"
      "Options:\n"
      "  -E1   enable E test\n"
      "  -E0   disable E test\n"
      "  -CM1  enable CM test\n"
      "  -CM0  disable CM test\n"
      "  -D1   enable D test\n"
      "  -D0   disable D test\n"
      "  -ED1  enable E=>D test\n"
      "  -ED0  disable E=>D test\n"
      "  -RLD1 enable RLD test\n"
      "  -RLD0 disable RLD test\n"
      "  -c    count number of initial ideals without embedded primes\n"
      "        and number of Cohen Macaulay initial ideals when using -CM1\n"
      "  -P    polygon mode, testing E=>D\n";
  }

  ZeroOneOption optionE;
  ZeroOneOption optionCM;
  ZeroOneOption optionD;
  ZeroOneOption optionED;
  ZeroOneOption optionRLD;
  SimpleOption optionCount;
  SimpleOption optionRandomPolygons;
  
public:
  ConjectureApplication():
    optionE("-E",true),
    optionCM("-CM",true),
    optionD("-D",true),
    optionED("-ED"),
    optionRLD("-RLD"),
    optionCount("-c"),
    optionRandomPolygons("-p")
  {
    registerOptions();
  }

  int main()
  {
    /*    bool optionEnabledE=true;
	  bool optionEnabledCM=true;
	  bool optionEnabledD=true;
	  bool optionEnabledEImpliesD=false;
	  bool optionEnabledRLD=false;
	  bool optionRandomPolygons=false;
	  bool optionCount=false;
    */
    /*   if(lp_soplexlinked)
	 fprintf(stderr,"SoPlex library is available.\n");
	 else
	 {
	 fprintf(stderr,"SoPlex library is not available.\n");
	 assert(0);
	 }
    */
    print_alfabeta();
    
    
    if(optionRandomPolygons.getValue())
      {
	runRandomTests();
      }
    else
      {
	basis_i inputMatrix=toIndexedMatrix(FileParser(stdin).parseMatrix());
	
	testConjectures(inputMatrix,optionE.getValue(),optionCM.getValue(),optionD.getValue(),optionED.getValue(),optionRLD.getValue(),optionCount.getValue());
      }
    
    return 0;
  }
};

static ConjectureApplication theApplication;
