#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <vector>
#include <list>

using namespace std;


#include "vektor.h"
#include "lll.h"
#include "print.h"
#include "division.h"
#include "buchberger.h"
#include "toric.h"
#include "flip.h"
#include "lp.h"
#include "parser.h"
#include "radical.h"
#include "printer.h"
#include "macaulay2.h"
#include "decompose.h"
#include "facet.h"

#include "ep_latex.h"
#include "ep_tigers.h"

#include "ea_exhaustive.h"
#include "ea_reverse.h"
#include "ea_exhaustiveradical.h"
#include "ea_reverseradical.h"

#include "result.h"

#include "testresult.h"
#include "normaliz.h"
#include "xml.h"
#include "counter.h"

#include "application.h"

Macaulay2Pipe M2;























/*
static void printhelp(FILE *f=stdout)
{
  char *help="";

  fprintf(f,help);

  if(lp_soplexlinked)
    fprintf(f,"SoPlex library is available.\n");
  else
    fprintf(f,"SoPlex library is not available.\n");
}
*/

static void printInitialIdeal(gbasis const &g, int cc)
{
  fprintf(stdout,"Groebner basis:\n");
  AsciiPrinter(stdout).printGroebnerBasis(g);
  fprintf(stdout,"\nRadical:\n");
  AsciiPrinter(stdout).printGroebnerBasis(initialradical(g));
  fprintf(stdout,"\nStandard Pairs:\n");
  AsciiPrinter(stdout).printStandardPairList(computeStandardPairs(initialIdeal(g)));
      
  M2.setPolynomialRing(g);
  fprintf(stdout,"Macaulay2: pdim coker gens:%i ==? n-rank(A):%i\n",M2.getPdimCokerGensMonomial(initialIdeal(g)),cc);
  fprintf(stdout,"Degree: %i\n",degree(g));
}

  
bool runExample(vektor_i v, FILE *f, FILE *fxml)
{
  bool ret;
  int d=v.size();
  gbasis a;

  for(int i=0;i<d;i++)
    {
      if(i==d-1)a.push_back(v);
      else
	{
	  vektor_i u(d);
	  for(int j=0;j<d;j++)u[j]=j==i;
	  a.push_back(u);
	}
    }

  gbasis a2=normaliz(a);

  a2.push_front(vektor_i(d));

  for(gbasis::iterator i=a2.begin();i!=a2.end();i++)
    {
      vektor_i u(i->size()+1);
      for(int t=0;t<i->size();t++)u[t]=(*i)[t];
      u[i->size()]=1;
      *i=u;
    }

  fprintf(stdout,"Normaliz input:\n");
  AsciiPrinter(stdout).printVectorList(a);

  fprintf(stdout,"Normaliz input 2:");
  AsciiPrinter(stdout).printVectorList(a2);

  if(isNormal(a2))
    {
      AsciiPrinter(f).printVectorList(a2);

      int rankA=rank(toIndexedMatrix(a2));
      gbasis g=toric1(toIndexedMatrix(a2));

      M2.setPolynomialRing(g);

      printf("Changing term order to rev lex with $a<b<c....$\n\n");

      vektor_i::setcheapest2(0,a2.size());
      //  vektor_i::setcheapest(a2.size()-1);
      vektor_i::TERMORDER=REVERSELEXICOGRAPHIC;
      buchberger(g);
      gbasisminimal(&g);
      gbasisautoreduction(&g);
      int n=g.begin()->size();
      
      fprintf(stdout,"Searching....\n");
      int counter=0;
      bool chooseBetter=true;
      bool foundE=false;
      bool foundCM=false;
      bool foundD=false;
      gbasis basisE;
      gbasis basisCM;
      gbasis basisD;

      do
	{
	  gbasis facets=irredundant(g);
	  
	  int toDo=rand()&3;
	  
	  gbasis relevant;
	  
	  if((rand()&15)==0)
	    chooseBetter=(toDo<3);
	  
	  if(chooseBetter)
	    {
	      //pick random better ideal
	      StandardPairList spl=computeStandardPairs(g);
	      int best=spl.size();
	      {
		if((counter&1023)==0)
		  {
		    fprintf(stderr,"Number of standard pairs: %i\n",spl.size());
		  }
		counter++;
	      }
	      for(gbasis::const_iterator i=facets.begin();i!=facets.end();i++)
		{
		  gbasis n=flip(g,*i);
		  gbasisminimal(&n);
		  gbasisautoreduction(&n);
		  
		  StandardPairList spl2=computeStandardPairs(n);
		  
		  if(true)
		    {
		      if(spl2.size()<best)
			{
			  relevant.push_back(*i);
			  best=spl2.size();
			}
		    }
		  else
		    {
		      if(spl2.size()<spl.size())
			{
			  relevant.push_back(*i);
			}
		    }
		}
	    }
	  else
	    {
	      // pick random
	      relevant=facets;
	    }
	  
	  if(relevant.size()==0)relevant=facets;
	  
	  assert(relevant.size()>0);
	  int index=((unsigned int)rand())%relevant.size();
	  
	  gbasis::const_iterator j=relevant.begin();
	  while(index--)j++;
	  vektor_i flipBinomial=*j;
	  
	  gbasis ny=flip(g,flipBinomial);
	  gbasisminimal(&ny);
	  gbasisautoreduction(&ny);
	  
	  g=ny;
	  
	  if((!foundCM)&&(!foundE))
	    if(!hasEmbeddedPrimes(initialIdeal(g)))
	      {
		if(!foundCM)
		  {
		    if(M2.getPdimCokerGensMonomial(initialIdeal(g))==(n-rankA))
		      {
			fprintf(stdout,"Found Cohen-Macaulay initial ideal.\n");
			fprintf(stderr,"Found Cohen-Macaulay initial ideal.\n");
			basisCM=g;
			foundCM=true;
		      }
		  }
		if(!foundE)
		  {
		    fprintf(stdout,"Found initial ideal without embedded primes.\n");
		    fprintf(stderr,"Found initial ideal without embedded primes.\n");
		    basisE=g;
		    foundE=true;
		  }
	      }
	  if(!foundD)
	    {
	      if(degree(g)<=rankA)
		{
		  fprintf(stdout,"Found initial ideal with low degree.\n");
		  fprintf(stderr,"Found initial ideal with low degree.\n");
		  basisD=g;
		  foundD=true;
		}
	    }

	}
      while((!foundE)||(!foundD)||(!foundCM));
      
      fprintf(stdout,"Initial ideal without embedded primes:\n");
      printInitialIdeal(basisE,n-rankA);
      fprintf(stdout,"Cohen-Macaulay initial ideal:\n");
      printInitialIdeal(basisCM,n-rankA);
      fprintf(stdout,"Initial ideal with low degree.\n");
      printInitialIdeal(basisD,n-rankA);
      ret=true;
    }
  else
    {
      printf("Skipping test - extended Firla-Ziegler example was not normal\n"); 
      ret=false;
    }
  fflush(stdout);

  return ret;
}

bool stringSearch(FILE *f, char *s)
{
  // no duplicates in s!!!!!
  int pos=0;
  int l=strlen(s);

  while(!feof(f) && pos<l)
    {
      int c=getc(f);

      if(c==s[pos])
	pos++;
      else
	{
	  if(pos)ungetc(c,f);
	  pos=0;
	}
    }
  return pos==l;
}


void runExamples(char *s)
{
  FILE *fxml =fopen("FZexamples.xml","w");
  FILE *f2=fopen("FZexamples-normal.txt","w");

  int n=0;
  int numberOfNormalExamples=0;
  int numberOfSkippedExamples=0;

  FILE *f;
  f=fopen(s,"r");
  assert(f);
  while(stringSearch(f,"CONE")&& n++<370)//370
    {
      char temp[1024];
      fgets(temp,1024,f);
      int l=strlen(temp);
      for(int i=0;i<l-1;i++)
	{
	  if(temp[i]==' ' || temp[i]=='\t')
	    {
	      if(temp[i-1]>='0' && temp[i-1]<='9')temp[i]=',';
	    }
	  if(temp[i]==')')temp[i+1]=0;
	}
      //      vektor_i v(temp);
      vektor_i v=StringParser(temp).parseIntegerVector();//24. september

      fprintf(stdout,"------------------------------------------------------------\n");
      fprintf(stdout,"          Example no. %i (%i counting from 1)\n",n-1,n);
      fprintf(stderr,"          Example no. %i (%i counting from 1)\n",n-1,n);
      fprintf(stdout,"------------------------------------------------------------\n");

      if(runExample(v,f2,fxml))
	numberOfNormalExamples++;
      else
	numberOfSkippedExamples++;
    }
  fclose(f2);
  fclose(fxml);

  fprintf(stdout,"Total: %i / Normal: %i / Skipped: %i\n",numberOfSkippedExamples+numberOfNormalExamples,numberOfNormalExamples,numberOfSkippedExamples); 
}


//--------------------------------------------------
// Main
//--------------------------------------------------

class FirlaZieglerApplication : public CatsApplication
{
  const char *helpText()
  {
    return "This is a program for testing conjectures on the Firla-Ziegler examples.\n"
      "Options: -d4 Test four dimensional examples reading 4_dim_no_part_small.txt\n";
      "         -d5 Test four dimensional examples reading 5_dim_no_part_small.txt\n";
  }

  IntegerOption optionDimension;

public:
  FirlaZieglerApplication():
    optionDimension("-d",4,4,5)
  {
    registerOptions();
  }

  int main()
  {
    /*    if(lp_soplexlinked)
      fprintf(stderr,"SoPlex library is available.\n");
    else
      {
	fprintf(stderr,"SoPlex library is not available.\n");
	assert(0);
      }
    */
    print_alfabeta();
    
    char *filename;
    if(optionDimension.getValue()==4)
      filename="4_dim_no_part_small.txt";
    else
      filename="5_dim_no_part_small.txt";
    
    fprintf(stdout,"example filename:%s\n",filename);
    fprintf(stderr,"example filename:%s\n",filename);
    runExamples(filename);
  }
};

static FirlaZieglerApplication theApplication;



  
