#include "normaliz.h"
#include "parser.h"
#include "printer.h"
#include "lll.h"

#include "application.h"

class IsDeltaNormalApplication : public CatsApplication
{
public:
  SimpleOption optionAllTriangulations;
  SimpleOption optionVerbose;
  
  IsDeltaNormalApplication():
    optionAllTriangulations("-a"),
    optionVerbose("-v")
  {
    registerOptions();
  }


  const char *helpText()
  {
    return "This program tests if a given point configuration\n"
      "has a regular triangulation delta for which it is delta normal.\n"
      "This program uses points2triangs and normaliz which should be in the path.\n"
      "The cone spanned by the point configuration should not contain a 1-dimensional subspace.\n"
      "Delta-normality is checked with respect to the lattice generated by the\n"
      "point configuration. So the the first thing this program does is changing\n"
      "the coordinates of the points to coordinates in a lattice basis of the\n"
      "generated lattice.\n"
      "The input to the program is a matrix in CaTS' format.\n"
      "Options:\n"
      "  -a  use all connected triangulations, not only regular ones.\n"
      "      This is faster but if you get the answer \"true\" you are\n"
      "      NOT sure that the delta is a regular triangulation. So the\n"
      "      right answer could be \"false\"!\n"
      "  -v  Print comments\n";
  }

  
  int main()
  {
    //    bool printComments=false;
    //    bool onlyRegular=true;
    
    gbasis t;
    gbasis input=FileParser(stdin).parseMatrix();

    if(1)
      {
	fprintf(stderr,"Changing coordinates to lattice basis coordinates...\n");
	fprintf(stderr,"Original point configuration:\n");
	AsciiPrinter(stderr).printVectorList(input);
	basis_i inverseTransposedM;
	basis_i latticeBasis=toIndexedMatrix(input);
	basis_i m=mlll(latticeBasis,&inverseTransposedM);
	fprintf(stderr,"Lattice basis:\n");
	AsciiPrinter(stderr).printVectorList(toLinkedMatrix(latticeBasis));
	int d=rank(toIndexedMatrix(input));
	gbasis newConfiguration;
	for(int i=0;i<inverseTransposedM[0].size();i++)
	  {
	    vektor_i v(d);
	    for(int j=0;j<d;j++)v[j]=inverseTransposedM[inverseTransposedM.size()-d+j][i];
	    newConfiguration.push_back(v);
	  }
	input=newConfiguration;
	fprintf(stderr,"New point configuration:\n");
	AsciiPrinter(stderr).printVectorList(input);
      }
    
    
    if(1)
      {
	fprintf(stderr,"just running normaliz for fun....\n");
	gbasis result=normaliz(input);
	fprintf(stderr,"Result:\n");
	AsciiPrinter(stderr).printVectorList(result);
	fprintf(stderr,"\n");
      }
    
    bool result=isDeltaNormal(toIndexedMatrix(input),&t,optionAllTriangulations.getValue(),optionVerbose.getValue()?2:1);
    printf(result?"true":"false");
    printf("\n");
    
    if(result)
      {
	AsciiPrinter(stdout).printVectorList(t);
	printf("\n");
      }
    
    return 0;
  }
};

static IsDeltaNormalApplication theApplication;
