#include "dprint.h"
#include "radical.h"
#include "facet.h"

//-----------------------------------------------------------------
// Detailed Printing
//-----------------------------------------------------------------

bool vektorcmp(const vektor_i& a, const vektor_i& b)
{
  if(a.size()!=b.size())return false;
  for(int i=0;i<a.size();i++)
    {
      if(a[i]!=b[i])return false;
    }
  return true;
}

bool gbasiscmp(const gbasis& a,const gbasis& b)
{
  gbasis::const_iterator i=a.begin();
  gbasis::const_iterator j=b.begin();

  while((i!=a.end())&&(j!=b.end()))
    {
      if(!vektorcmp(*i,*j))return false;

      i++;
      j++;
    }

  if((i==a.end())&&(j==b.end()))return true;

  return false;
}

bool gfanmember(const gbasis& b, gfan f)
{
  for(gfan::const_iterator i=f.begin();i!=f.end();i++)
    {
      if(gbasiscmp(*i,b))return true;
    }
  return false;
}

static bool ismember(const vektor_i &p, const gbasis &b)
{
  for(gbasis::const_iterator i=b.begin();i!=b.end();i++)
    {
      vektor_i a=*i-p;
      if(a.iszero())return true;
    }
  return false;
}

static int getindex(const gbasis &b, const gfan &f)
{
  int index=0;

  for(gfan::const_iterator i=f.begin();i!=f.end();i++)
    {
      if(gbasiscmp(*i,b))return index;
      index++;
    }
  return index;
}

static void printt3(const vektor_i &v, FILE *f=stdout)
{
  bool a=v.ordered();

  print_math(f);
  if(a)fprintf(f,"\\underline{");
  printmono(v,1,f);
  if(a)fprintf(f,"}");
  fprintf(f,"-");
  if(!a)fprintf(f,"\\underline{");
  printmono(v,-1,f);
  if(!a)fprintf(f,"}");
  print_math(f);
  fprintf(f,"\n\n");
}


bool lexicographiccmp(vektor_i v)
{
  int n=v.size();
  for(int i=0;i<n;i++)
    {
      int index=i;
      if(v[index]<0)return 0;
      if(v[index]>0)return 1;
    }
  return 1;
}

void printt3(const gbasis &b, const gfan &gf, FILE *f=stdout)
{
  gbasis irr=irredundant(b);
  int best=-1;
  const vektor_i *bestref;
  int ind=0;
  

  for(gbasis::const_iterator i=b.begin();i!=b.end();i++)
    {
      if(ismember(*i,irr))
	{
	  if(!i->ordered())
	    {
	      if(best==-1)
		{
		  best=ind;
		  bestref=&(*i);
		}
	      else
		{
		  vektor_i a=*i;
		  vektor_i b=*bestref;
		  
		  //		  a.order();
		  //		  b.order();
		  
		  //		  if(lexicographiccmp(b.lt()-a.lt()))
		  if(lexicographiccmp(a.lt()-b.lt())) //rettet 18.april
		    {
		      best=ind;
		      bestref=&(*i);
		    }
		}
	    }
	}
      ind++;
    }

  ind=0;

  for(gbasis::const_iterator i=b.begin();i!=b.end();i++)
    {
      if(!ismember(*i,irr))
	{
	  printt3(*i);
	}
      else
	{
	  gbasis beta=flip(b,*i);
	  //	  printf("resultat af flip:\n\n");printt(beta);
	  gbasisminimal(&beta);
	  gbasisautoreduction(&beta);
	  gbasissort(&beta);

	  int index=getindex(beta,gf);

	  if(ind==best)printf("\\fbox{");
	  printf("\\fbox{");
	  printt3(*i);
	  if(ind==best)printf("}");
	  printf("}$_{%i}$\n\n",index);
	}
      ind++;
    }
}

void gfanprint(const gfan &B)
{
  {
    int j=0;
    for(gfan::const_iterator i=B.begin();i!=B.end();i++)
      {
	printf("\\fbox{\n");
	printf("Basis no. %i\n\n",j);
	printt3(*i,B);     
	printf("}\n\n");
	if(0)
	  {
	    printf("\\fbox{\n");
	    printf("Radical %i\n\n",j);
	    printinitial(initialradical(*i));     
	    printf("}\n\n");
	  }
	j++;
      }
  }
}

