#include "ea_exhaustive.h"
#include "division.h"
#include "flip.h"
#include "facet.h"
#include "printer.h"
#include "walk.h"

#include <set>

//--------------------------------------------------
// ExhaustiveSearch
//--------------------------------------------------

gbasis ExhaustiveSearch::myFlip(const gbasis& g, const vektor_i& p)
{
  gbasis beta=flip(g,p);
  gbasisminimal(&beta);
  gbasisautoreduction(&beta);
  return beta;
}

bool ExhaustiveSearch::flipable(const vektor_i &v)
{
  return (!useOrdered)||(v.ordered());
}


gbasis ExhaustiveSearch::findRoot(gbasis const &g)
{
  return walk(g);
}

void ExhaustiveSearch::enumerate(gbasis g)
{
  int nflips=0;
  gfan2 A,B;
  int Asize=0;

  if(useOrdered)g=findRoot(g);

  targetBeginEnumeration(g);

  gbasissort(&g);
  A.insert(g);
  Asize++;
  B.insert(g);

  while(A.size()>0)
    {
      gbasis b=*A.begin();
      targetBasis(b);

      A.erase(A.begin());
      Asize--;

      for(gbasis::iterator i=b.begin();i!=b.end();i++)i->calcsupport();
      
      gbasis irrData;
      const gbasis &irr=(allFlipable)?algebraicFacets(b):irrData=irredundant(b);
      {
	static int i;
	i++;
	if(!(i&255))
	  fprintf(stderr,"Number of active vertices:%i. Number of facets for current cone:%i. Current total number of Gr\"obner bases: %i\n",Asize,irr.size(),B.size());
      }
      for(gbasis::const_iterator i=irr.begin();i!=irr.end();i++)
	if(flipable(*i))
	{
	  gbasis beta=myFlip(b,*i);
	  nflips++;
	  gbasissort(&beta);
	  if(insertionTest(*i))
	    if(B.count(beta)==0)
	      {
		A.insert(beta);
		Asize++;
		B.insert(beta);
	      }
	}
    }

  result->numberOfVertices.assign(B.size());
  result->numberOfEdges.assign(useOrdered?nflips:nflips/2);

  targetEndEnumeration();
}

//--------------------------------------------------
// ExhaustiveSearchAGraded
//--------------------------------------------------

bool AreBinomialSetsEqual(const gbasis &a, const gbasis &b)
{
  gbasis A=a;
  gbasis B=b;

  A.sort();
  B.sort();
  
  gbasis::iterator i=A.begin();
  gbasis::iterator j=B.begin();
  while(i!=A.end() && j!=B.end())
    {
      if(*j!=*i)return false;
      i++;
      j++;
    }

  return (i==A.end())==(j==B.end());
}

ExhaustiveSearchAGraded::ExhaustiveSearchAGraded(const basis_i &A):
  useGraverBasis(true)
{
  setAllFlipable(true);
  setUseOrdered(false);
  graver=GraverBasis(A);
}

ExhaustiveSearchAGraded::ExhaustiveSearchAGraded(const gbasis &trueGroebnerBasis):
  useGraverBasis(false)
{
  setAllFlipable(true);
  setUseOrdered(false);
  this->trueGroebnerBasis=trueGroebnerBasis;
}

gbasis ExhaustiveSearchAGraded::myFlip(const gbasis& g, const vektor_i& p)
{
  gbasis beta;
  if(useGraverBasis)
    beta=flipAGraded(g,p,graver);
  else
    beta=flipAGraded(g,p,trueGroebnerBasis);
  gbasisminimal(&beta);
  return beta;
}

