#include "ea_hybrid.h"
#include "facet.h"

//-----------------------------------------------------------------
// Hybrid
//-----------------------------------------------------------------

Hybrid::Hybrid():
  numberOfEdges(0),
  numberOfVertices(0)
{
}

static bool vektorcmp2(const vektor_i& a, const vektor_i& b)
{
  //  if(a.size()!=b.size())return false;
  for(int i=0;i<a.size();i++)
    {
      if(a[i]<b[i])return true;
      if(a[i]>b[i])return false;
    }
  return false;
}

struct it;

static it *new_it();
static void delete_it(it*);

typedef struct pgb
{
  gbasis gb;
  int priority;
  gbasis outedges;
  it *i;
  //  pgbqueue::iterator i;
  pgb():gb(),outedges()
  {
    i=new_it();
  }
  ~pgb()
  {
    delete_it(i);
  }

  void computefacets()
  {
    gbasis irr=irredundant(gb);
    for(gbasis::const_iterator i=irr.begin();i!=irr.end();i++)
      {
	if(i->ordered())
	  {
	    outedges.push_back(*i);
	  }
	else
	  {
	    priority++;
	  }
      }
    //    fprintf(stderr,"computed priority:%i\n",priority);
  }
}pgb;

struct hbgbcmp1
{
  //bool operator()(const pgb* &a,const pgb* &b) const
bool operator()(pgb* const &a,pgb* const &b) const
{
  gbasis::const_iterator i=a->gb.begin();
  gbasis::const_iterator j=b->gb.begin();

  while((i!=a->gb.end())&&(j!=b->gb.end()))
    {
      if(vektorcmp2(*i,*j))return true;
      if(vektorcmp2(*j,*i))return false;

      i++;
      j++;
    }
  if(i!=a->gb.end())return false;
  if(j!=b->gb.end())return true;

  return false;
}
};

struct hbgbcmp2
{
bool operator()(pgb* const &a, pgb* const &b) const
{
  if(a->priority<b->priority)return true;
  if(a->priority>b->priority)return false;
  gbasis::const_iterator i=a->gb.begin();
  gbasis::const_iterator j=b->gb.begin();

  while((i!=a->gb.end())&&(j!=b->gb.end()))
    {
      if(vektorcmp2(*i,*j))return true;
      if(vektorcmp2(*j,*i))return false;

      i++;
      j++;
    }
  if(i!=a->gb.end())return false;
  if(j!=b->gb.end())return true;

  return false;
}
};

typedef set<pgb*,hbgbcmp1> pgbset;
typedef set<pgb*,hbgbcmp2> pgbqueue;

struct it
{
  pgbqueue::iterator i;
};

it* new_it()
{
  return new it;
}

void delete_it(it* i)
{
  delete i;
}


void Hybrid::enumerate(gbasis g)
{
  numberOfVertices=0;
  numberOfEdges=0;

  int fansize=0;
  pgbset s;
  pgbqueue q;

  targetBeginEnumeration(g);

  gbasisminimal(&g);
  gbasisautoreduction(&g);
  gbasissort(&g);


  pgb *a=new pgb;
  pgb *temppgb=new pgb;

  a->gb=g;
  a->priority=0;
  a->computefacets();

  s.insert(a);
  a->i->i=q.insert(q.begin(),a);

  while(!q.empty())
    {
      numberOfVertices++;
      a=*q.begin();
      targetBasis(a->gb);
      
      q.erase(q.begin());
      s.erase(s.find(a));

      assert(a->priority==0);
      for(gbasis::const_iterator i=a->outedges.begin();i!=a->outedges.end();i++)
	{
	  numberOfEdges++;
	    {
	      gbasis temp=flip(a->gb,*i);
	      gbasisminimal(&temp);
	      gbasisautoreduction(&temp);
	      gbasissort(&temp);

	      temppgb->gb=temp;
	      if(s.end()==s.find(temppgb))
		{
		  pgb *b=new pgb;
		  b->gb=temp;

		  b->priority=-1;
		  b->computefacets();
		  
		  s.insert(b);
		  b->i->i=q.insert(q.begin(),b);
		}
	      else
		{
		  pgbset::iterator j=s.find(temppgb);
		  q.erase((*j)->i->i);
		  (*j)->priority--;
		  (*j)->i->i=q.insert(q.begin(),*j);
		}
	    }	  
	}
      delete a;
    }
  result->numberOfVertices.assign(numberOfVertices);
  result->numberOfEdges.assign(numberOfEdges);

  delete temppgb;

  targetEndEnumeration();
}

