#ifndef ENUMERATION_H_INCLUDED
#define ENUMERATION_H_INCLUDED

#include <string>
#include "vektor.h"
#include "result.h"

class EnumerationTarget
{
 public:
  virtual void beginEnumeration(const gbasis &g)=0;
  virtual void endEnumeration()=0;
  virtual void beginRadical(const gbasis &radical, int numberOfSymmetries=0)=0;
  virtual void endRadical()=0;
  virtual bool basis(const gbasis &g)=0; /* return false to break enumaration */
};


class EnumerationPrinter: public EnumerationTarget
{
};


class EnumerationFilePrinter: public EnumerationPrinter
{
  FILE *initialisedFile;
 protected:
  string filename;
  FILE *file;
 public:
  EnumerationFilePrinter();
  ~EnumerationFilePrinter();

  void open(std::string filename);
  void open(FILE *file);
  void close();

  virtual void onOpened()=0;
  virtual void onClose()=0;
  virtual void onClosed()=0;

  virtual string extension();
};


class EnumerationAlgorithm
{
 protected:
  Result dummyResult;
  Result *result;
  EnumerationTarget *target;
  int progressCounter;
  virtual void printProgress(int step=1);
  void targetBeginEnumeration(const gbasis &g){if(target)target->beginEnumeration(g);}
  void targetEndEnumeration(){if(target)target->endEnumeration();}
  void targetBeginRadical(const gbasis &radical){if(target)target->beginRadical(radical,0);}
  void targetEndRadical(){if(target)target->endRadical();}
  bool targetBasis(const gbasis &g){bool ret=true;if(target)ret=target->basis(g);printProgress();return ret;}
 public:
  EnumerationAlgorithm():result(&dummyResult){target=0;progressCounter=0;}
  void setEnumerationTarget(EnumerationTarget *target){this->target=target;}
  void setResult(Result *result){this->result=result;}
  virtual void enumerate(gbasis g){}
};

#endif
