#include "ep_tigers.h"
#include "print.h"
#include "printer.h"


TigersEnumerationPrinter::TigersEnumerationPrinter():
  printer(0),
  initialIdealPrinting(false),
  stripPrinting(false),
  radicalCounter(0),
  numberOfBegins(0),
  insertComma(false)
{
}


void TigersEnumerationPrinter::initialIdealPrintingSetEnabled(bool enabled)
{
  initialIdealPrinting=enabled;
}


void TigersEnumerationPrinter::stripPrintingSetEnabled(bool enabled)
{
  stripPrinting=enabled;
}


void TigersEnumerationPrinter::onOpened()
{
  if(stripPrinting)
    fprintf(file,"{");
}


void TigersEnumerationPrinter::onClose()
{
  if(stripPrinting)
    fprintf(file,"}");
}


void TigersEnumerationPrinter::onClosed()
{
}


void TigersEnumerationPrinter::beginEnumeration(const gbasis &g)
{
  if(numberOfBegins==0)
    {
      basisCounter=0;
    }
  numberOfBegins++;
}


void TigersEnumerationPrinter::endEnumeration()
{
  numberOfBegins--;
  if(numberOfBegins==0)
    {
    }
}


void TigersEnumerationPrinter::beginRadical(const gbasis& radical, int numberOfSymmetries)
{
  firstBasisInThisRadical=basisCounter;
  fprintf(file,"Radical %i: ",radicalCounter++);
  print_tigers_in_strip(radical,0,file);
  fprintf(file,"\n");
  if(numberOfSymmetries)
    {
      fprintf(file,"Number of known symmetries: %i\n",numberOfSymmetries);
    }
  fprintf(file,"{");
  insertComma=false;
}


void TigersEnumerationPrinter::endRadical()
{
  fprintf(file,"}\n");
  fprintf(file,"Number of initial ideals with this radical: %i\n\n",basisCounter-firstBasisInThisRadical);
}


bool TigersEnumerationPrinter::basis(const gbasis &g)
{
  if(initialIdealPrinting)
    {
      if(!stripPrinting)
	print_tigers_in(g,basisCounter,file);
      else
	{
	  if(insertComma)fprintf(file,",\n");
	  print_tigers_in_strip(g,basisCounter,file);
	  insertComma=true;
	}
    }
  else
    {
      if(!stripPrinting)
	{ // option -p1
	  print_tigers(g,basisCounter,file);
	  print_tigers_strip(g,basisCounter,file);
	  fprintf(file,"\n");
	}
      else
	{ // option -p1+
	  if(insertComma)fprintf(file,",\n");
	  print_tigers_strip(g,basisCounter,file);
	  insertComma=true;
	}
    }
  basisCounter++;

  return true;
}


string TigersEnumerationPrinter::extension()
{
  //  return ".list";
  return "";
}
