#include "facet.h"
#include "lp.h"
#include "printer.h"
#include "counter.h"

//#include "main.h"

static Timer superSetTimer("Superset-timer"); 

static bool supersetEnabled=true;

void setSupersetEnabled(bool enabled)
{
  supersetEnabled=enabled;
}

//-----------------------------------------------------------------
// Facet Detection
//-----------------------------------------------------------------
bool isAlgebraicFacet(const gbasis &g, const gbasis::const_iterator &i)
{
  TimerScope ts0(&superSetTimer);
  // This is the superset test.
  // Note! being an "algebraicFacet" is not the same as being a geometric facet.

  /*  printf("is algebraicFace():\n");
  AsciiPrinter(stdout).printGroebnerBasis(g);
  printf("\n");
  AsciiPrinter(stdout).printBinomial(*i);
  printf("\n");
  */


  assert(supersetEnabled);

  if(!supersetEnabled)return true;
  
  if(i->tl().iszero())return false; // This is only the case if the ideal is not homogeneous with respect to any positive grading

  bool facet=true;
  vektor_i s(i->size());
  for(gbasis::const_iterator j=g.begin();j!=g.end();j++)
    {
      bool inideal=false;
      if(i!=j)
	{
	  for(int k=0;k<s.size();k++)s[k]=max(max((*i)[k],0),(*j)[k])-(*i)[k];
	  s.calcsupport();
	  //AsciiPrinter(stdout).printBinomial(s);

	  for(gbasis::const_iterator l=g.begin();l!=g.end();l++)
	    if(((l->support)&(~s.support))==0)
	      if(l->divides(s))
		{
		  inideal=true;
		  break;
		}
	  if(!inideal)
	    {
	      facet=false;
	      break;
	    }
	}
    }
  //  printf("ret:%i\n",facet);

  return facet;
}


gbasis algebraicFacets(const gbasis g)
{
  gbasis f;

  for(gbasis::const_iterator i=g.begin();i!=g.end();i++)
    if(isAlgebraicFacet(g,i))f.push_back(*i);

  return f;
}


gbasis irredundant(const gbasis &g)
{
  gbasis temp=supersetEnabled ? algebraicFacets(g) : g;

  //  fprintf(stderr,"A:%i B:%i\n",g.size(),temp.size());

  gbasis r;

  for(gbasis::const_iterator i=temp.begin();i!=temp.end();i++)
    {
      if(isfacet(temp,i))
	 r.push_back(*i);
    }
  return r;
}

