#include "fiber.h"

#include "printer.h"

Fiber::Iterator::Iterator(const gbasis &gb, const vektor_i rootElement):
  g(&gb)
{
  StackElement a;
  a.v=rootElement;
  a.i=g->begin();
  stack.push_front(a);
}

  
bool Fiber::Iterator::isEdge(const vektor_i &v, gbasis::const_iterator &i)
{
  if((-(*i)).divides(v))
    {
      for(gbasis::const_iterator j=g->begin();j!=i;j++)
	{
	  if(j->divides(v+*i))return false;
	}
      return true;
    }
  else
    return false;
}


bool Fiber::Iterator::operator++()
{
  while(1)
    {
      //      printStack();
      while(stack.begin()!=stack.end() && stack.begin()->i==g->end())
	{
	  stack.pop_front();
	  if(stack.begin()!=stack.end())stack.begin()->i++;
	}
      if(stack.begin()!=stack.end())
	{
	  if(isEdge(stack.begin()->v,stack.begin()->i))
	    {
	      StackElement a;
	      a.v=stack.begin()->v+*(stack.begin()->i);
	      a.i=g->begin();
	      stack.push_front(a);
	      break;
	    }
	  else
	    stack.begin()->i++;
	}
      else
	break;
    }
};


const vektor_i& Fiber::Iterator::operator*()
{
  assert(stack.begin()!=stack.end());
  return stack.begin()->v;
}


void Fiber::Iterator::printStack()
{
  fprintf(stderr,"Printing stack: size=%i\n",stack.size());
  for(list<StackElement>::iterator i=stack.begin();i!=stack.end();i++)
    {
      fprintf(stderr,"monomial: ");
      AsciiPrinter(stderr).printMonomial(i->v);
      fprintf(stderr,"\niterator: ");
      if(i->i!=g->end())
	AsciiPrinter(stderr).printBinomial(*(i->i));
      else
	fprintf(stderr,"end");
      fprintf(stderr,"\n");
    }
  fprintf(stderr,"\n");
}


Fiber::Fiber(const gbasis &gb, const vektor_i &fiberElement):g(gb)
{
  rootElement=fiberElement;division_(&rootElement,g);
  //  AsciiPrinter(stderr).printBinomial(rootElement);
}


Fiber::Iterator Fiber::begin()
{
  Iterator ret(g,rootElement);
  return ret;
}


Fiber::Iterator Fiber::end()
{
  Iterator ret;
  return ret;
}


void Fiber::print(Printer &p)
{
  bool first=true;

  p.printString("{\n");
  for(Iterator j=begin();j!=end();++j)
    {
      if(!first)
	p.printString(string(",\n"));
      p.printVector(*j);
      first=false;
    }
  p.printString("\n}\n");
}


void liftUsingFiber(gbasis *gLifting, const gbasis &g)
{
  for(gbasis::iterator i=gLifting->begin();i!=gLifting->end();i++)
    {
      Fiber f(g,*i);
      Fiber::Iterator j;
      for(j=f.begin();j!=f.end();++j)
	{
	  if(!isLeadingTermInInitialIdeal(*j,*gLifting))
	    {
	      *i-=*j;
	      break;
	    }
	}
      assert(j!=f.end());
    }
}
