#include "flip.h"
#include <stdio.h>
#include <stdlib.h>
#include "lp.h"
#include "radical.h"
#include "dprint.h"
#include "facet.h"
#include "graver.h"
#include "printer.h"
#include "counter.h"
#include "fiber.h"


static Timer flipTimer("Flip-timer"); 

//-----------------------------------------------------------------
// Flip
//-----------------------------------------------------------------

bool vektorcmp2(const vektor_i& a, const vektor_i& b)
{
  //  if(a.size()!=b.size())return false;
  for(int i=0;i<a.size();i++)
    {
      if(a[i]<b[i])return true;
      if(a[i]>b[i])return false;
    }
  return false;
}

bool gbcmp::operator()(const gbasis& a,const gbasis& b) const
{
  gbasis::const_iterator i=a.begin();
  gbasis::const_iterator j=b.begin();

  while((i!=a.end())&&(j!=b.end()))
    {
      if(vektorcmp2(*i,*j))return true;
      if(vektorcmp2(*j,*i))return false;

      i++;
      j++;
    }
  if(i!=a.end())return false;
  if(j!=b.end())return true;

  return false;
}


static void buchberger_(gbasis &spolys, vektor_i a)
{
  gbasis final;
  int totalspolys=0;
  vektor_i w=a;

  //  fprintf(stderr,"starting buchberger_\n");

  a.calcsupport();  // SUPPORT!!

  while(!spolys.empty())
    {
      vektor_i v=*spolys.begin();

      v.calcsupport();//SUPPORT!!

      totalspolys++;
      spolys.pop_front();
      {
	for(gbasis::const_iterator i=final.begin();i!=final.end();++i)
	  {
	    if(((i->support)&(~v.support))==0)
	      if(i->divides(v))goto next;
	  }
	//	while(a.lt().divides(v)){v=v-a;v.calcsupport();}//SUPPORT!!
	while(a.divides(v)){v-=a;}
	v.calcsupport();//SUPPORT!!
	for(gbasis::const_iterator i=final.begin();i!=final.end();++i)
	  {
	    if(((i->support)&(~v.support))==0)
	      if(i->divides(v))goto next;
	  }
      }
      
      if(!v.iszero())
	{
	  for(int i=0;i<w.size();i++)w[i]=max(max(a[i],0),v[i])-a[i];
	  w.calcsupport();//SUPPORT!!!
	  spolys.push_back(w);

	  v.calcsupport();
	  final.push_back(v);
	}
    next:;
    }

  // compute the minimal (and reduced!) basis

  for(gbasis::iterator i=final.begin();i!=final.end();i++)
    {
      bool e=true;
      for(gbasis::iterator j=final.begin();j!=final.end();j++)
	{
	  if(j!=i)
	    if(((j->support)&(~i->support))==0)
	      if(j->divides(*i)){e=false;break;}
	}
      if(e)spolys.push_back(*i);
    }
  //      spolys=final;
  //  fprintf(stderr,"ending buchberger_\n");

}

static int iszero(vektor_i v)
{
  for(int i=0;i<v.size();i++)if(v[i])return 0;
  return 1;
}

gbasis flip(const gbasis& g, const vektor_i& p)
{
  TimerScope ts0(&flipTimer);

  gbasis nevv;
  gbasis gm;

  //compute the monomials in new:
  for(gbasis::const_iterator i=g.begin();i!=g.end();++i)
    if(!(*i==p))nevv.push_back(i->lt());

  //compute basis for the initial ideal wrt other termorder
  buchberger_(nevv,-p);

  //the inverse to the flip binomial is in the new basis
  gm.push_back(-p);
  gm.back().calcsupport();//!!!!!support

  //nevv only contains monomials
  for(gbasis::const_iterator i=nevv.begin();i!=nevv.end();++i)
    if(iszero(*i-i->lt()))
      {
	vektor_i v=*i;

	v.calcsupport();//SUPPORT!!
	division_(&v,g);

	gm.push_back(*i-v);
	gm.back().calcsupport();//!!!!!support
      }

  return gm;
}

gbasis flipAGraded(const gbasis& g, const vektor_i& p, GraverBasis &graver)
{
  TimerScope ts0(&flipTimer);
  gbasis nevv;
  gbasis gm;

  //compute the monomials in new:
  for(gbasis::const_iterator i=g.begin();i!=g.end();++i)
    if(*i!=p)nevv.push_back(i->lt());

  //compute basis for the initial ideal wrt other termorder
  buchberger_(nevv,-p);

  //the inverse to the flip binomial is in the new basis
  gm.push_back(-p);
  gm.back().calcsupport();//!!!!!support

  //nevv only contains monomials
  for(gbasis::const_iterator i=nevv.begin();i!=nevv.end();++i)
    if(iszero(*i-i->lt()))
      {
	vektor_i v=*i;
	vektor_i v2;

	v.calcsupport();//SUPPORT!!

	GraverBasis::MatchIterator j;
	for(j=graver.matchBegin(v);j!=graver.matchEnd();++j)
	  {
	    if(!isLeadingTermInInitialIdeal(-*j,nevv)&&!p.tl().divides((*j).tl()))
	      {
		v2=*j;
		break;
	      }
	  }
	gm.push_back(v2);
	if(j==graver.matchEnd())
	  {
	    AsciiPrinter(stderr).printGroebnerBasis(nevv);
	    graver.print(stderr);
	  }
	assert(j!=graver.matchEnd());
	gm.back().calcsupport();//!!!!!support
      }

  return gm;
}

gbasis flipAGraded(const gbasis& g, const vektor_i& p, const gbasis& trueGroebnerBasis)
{
  TimerScope ts0(&flipTimer);
  /*
  fprintf(stderr,"flip groebnerbasis\n");
  AsciiPrinter(stderr).printGroebnerBasis(g);
  fprintf(stderr,"\nwith respect to:");
  AsciiPrinter(stderr).printBinomial(p);
  */
  gbasis nevv;
  gbasis gm;

  //compute the monomials in new:
  for(gbasis::const_iterator i=g.begin();i!=g.end();++i)
    if(*i!=p)nevv.push_back(i->lt());

  //compute basis for the initial ideal wrt other termorder
  buchberger_(nevv,-p);

  //the inverse to the flip binomial is in the new basis
  gm.push_back(-p);
  gm.back().calcsupport();//!!!!!support

  //nevv only contains monomials
  for(gbasis::const_iterator i=nevv.begin();i!=nevv.end();++i)
    if(iszero(*i-i->lt()))
      {
	vektor_i v=*i;
	vektor_i v2;

	v.calcsupport();//SUPPORT!!

	Fiber f(trueGroebnerBasis,v);
	Fiber::Iterator j;
	//	fprintf(stderr,"\nFiber:\n");
	for(j=f.begin();j!=f.end();++j)
	  {
	    //  AsciiPrinter(stderr).printBinomial(*j);
	    //fprintf(stderr,"\n");
	    if(!isLeadingTermInInitialIdeal(*j,nevv))
	      if(!p.tl().divides((*j).lt()))
		if(*j!=v)
		  {
		    v2=v-*j;
		    break;
		  }
	      
	  }
	gm.push_back(v2);
	if(j==f.end())
	  {
	    AsciiPrinter(stderr).printGroebnerBasis(nevv);
	    fprintf(stderr,"\nLifting:");
	    AsciiPrinter(stderr).printBinomial(v);
	    fprintf(stderr,"\nGrobnerbasis:");
	    AsciiPrinter(stderr).printGroebnerBasis(trueGroebnerBasis);
	  }
	assert(j!=f.end());
	gm.back().calcsupport();//!!!!!support
      }

  return gm;
}

