#include <assert.h>
#include <math.h>
#include <string.h>

#include "lp.h"
#include "print.h"
#include "counter.h"

LpSolver *LpSolver::list;

static bool soplexEnabled=true;
static bool subsetEnabled=true;


static Timer LPTimer("LP-timer"); // do this before soplex namespace 


int lp_status=0;

static inline bool subsettest(const gbasis &g, gbasis::const_iterator i)
{
  if(1)
    {
      //subset test

      for(int a=0;a<(*i).size();a++)
	{
	  bool ok=false;
	  if((*i)[a]>0)
	    {
	      gbasis::const_iterator b;
	      for(b=g.begin();b!=g.end();b++)
		if(b!=i)
		  if((*b)[a]>0)
		    {
		      ok=true;
		      break;
		    }
	    }
	  else
	    if((*i)[a]<0)
	      {
		gbasis::const_iterator b;
		for(b=g.begin();b!=g.end();b++)
		  if(b!=i)
		    if((*b)[a]<0)
		      {
			ok=true;
		      break;
		      }
	      }
	    else ok=true;
	  if(!ok)
	    {

	      if(0)
	      {
		gbasis G=g;
		printf("Skipping simplex with input:\n\n");
		printmatrix(G);
		print(*i);
	      }

	      return true;
	    }
	}
    }
  return false;
}


LpSolver::LpSolver()
{
   next=list;
   list=this;
}

LpSolver *LpSolver::find(const char *name)
{
   LpSolver *l=list;
   while(l)
      {
         if(string(l->name())==string(name))break;
         l=l->next;
      }
   return l;
}


void LpSolver::printList(FILE *f)
{
   fprintf(f,"List of linked LP solvers:\n");
   LpSolver *l=list;
   while(l)
      {
         fprintf(f," %s\n",l->name());
         l=l->next;
      }
}


static LpSolver *soplex,*huber,*cdd,*cddgmp,*default_;
static bool initialized;


bool lpSetSolver(const char *name)
{
  soplex=LpSolver::find("SoPlex");
  huber=LpSolver::find("Huber's");
  cdd=LpSolver::find("cdd");
  cddgmp=LpSolver::find("cddgmp");
  LpSolver *selected=LpSolver::find(name);
  default_=huber;
  if(soplexEnabled && soplex)default_=soplex;
  if(cdd)default_=cdd;
  if(selected)default_=selected;
  initialized=true;
  assert(default_);
  fprintf(stderr,"LP algorithm being used: \"%s\".\n",default_->name());

  return selected;
}

static void LpInit()
{
   if(!initialized)
      {
	lpSetSolver("");
      }
}

void setSubsetEnabled(bool enabled){subsetEnabled=enabled;}
void setSoplexEnabled(bool enabled){soplexEnabled=enabled;initialized=false;}

int isfacet(const gbasis &g, gbasis::const_iterator i)
{
   TimerScope ts0(&LPTimer);

   LpInit();

   if(subsetEnabled)
      if(subsettest(g,i))return true;

   return default_->isFacet(g,i); 
}
