#include "lp_cdd.h"
#include "setoper.h"
#include "cdd.h"
#include "cdd_f.h"

//--------------------------------------------------
// LpSolverCdd (double precision)
//--------------------------------------------------

static ddf_MatrixPtr vectorList2Matrix(const gbasis &g, ddf_ErrorType *Error)
{
  ddf_MatrixPtr M=NULL;
  ddf_rowrange m_input,i;
  ddf_colrange d_input,j;
  ddf_RepresentationType rep=ddf_Inequality;
  ddf_boolean found=ddf_FALSE, newformat=ddf_FALSE, successful=ddf_FALSE;
  char command[ddf_linelenmax], comsave[ddf_linelenmax];
  ddf_NumberType NT;

  (*Error)=ddf_NoError;

  rep=ddf_Inequality; newformat=ddf_TRUE;

  m_input=g.size();
  d_input=g.begin()->size();

  NT=ddf_Rational;

  M=ddf_CreateMatrix(m_input, d_input);
  M->representation=rep;
  M->numbtype=NT;

  gbasis::const_iterator I=g.begin();
  for (i = 0; i < m_input; i++) {
    for (j = 0; j < d_input; j++) {
      ddf_set_si(M->matrix[i][j],(*I)[j]);
    }
    I++;
  }
  
  successful=ddf_TRUE;

  return M;
}

static void cddinit()
{
  static bool initialized;
  if(!initialized)
    {
      ddf_set_global_constants();  /* First, this must be called. */
      initialized=true;
    }
}

bool LpSolverCdd::isFacet(const gbasis &g, gbasis::const_iterator i)
{
  bool ret;
  int index;
  ddf_MatrixPtr M=NULL,M2=NULL,M3=NULL;
  ddf_colrange d;
  ddf_ErrorType err=ddf_NoError;
  ddf_rowset redrows,linrows,ignoredrows, basisrows;
  ddf_colset ignoredcols, basiscols;
  //  long rank;
  mytype val;
  ddf_DataFileType inputfile;
  FILE *reading=NULL;


  cddinit();


  //  ddf_init(val);

  M=vectorList2Matrix(g, &err);

  if (err!=ddf_NoError) goto _L99;

  d=M->colsize;

  //  redrows=ddf_RedundantRows(M, &err);
  //  set_fwrite(stderr, redrows);


  index=0;
  for(gbasis::const_iterator J=g.begin();J!=g.end()&&J!=i;J++){index++;}
  if(index==g.size())assert(0);
  

  static ddf_Arow temp;

  ddf_InitializeArow(i->size(),&temp);

  ret= !ddf_Redundant(M,index+1,temp,&err);

  
  ddf_FreeMatrix(M);
  ddf_FreeArow(i->size(),temp);

  if (err!=ddf_NoError) goto _L99;
  return ret;
 _L99:
  assert(0);
  return false;

}

static LpSolverCdd theLpSolverCdd;


//--------------------------------------------------
// LpSolverCddGmp (exact arithmetics)
//--------------------------------------------------

static dd_MatrixPtr vectorList2MatrixGmp(const gbasis &g, dd_ErrorType *Error)
{
  dd_MatrixPtr M=NULL;
  dd_rowrange m_input,i;
  dd_colrange d_input,j;
  dd_RepresentationType rep=dd_Inequality;
  dd_boolean found=dd_FALSE, newformat=dd_FALSE, successful=dd_FALSE;
  char command[dd_linelenmax], comsave[dd_linelenmax];
  dd_NumberType NT;

  (*Error)=dd_NoError;

  rep=dd_Inequality; newformat=dd_TRUE;

  m_input=g.size();
  d_input=g.begin()->size();

  NT=dd_Rational;

  M=dd_CreateMatrix(m_input, d_input);
  M->representation=rep;
  M->numbtype=NT;

  gbasis::const_iterator I=g.begin();
  for (i = 0; i < m_input; i++) {
    for (j = 0; j < d_input; j++) {
      dd_set_si(M->matrix[i][j],(*I)[j]);
    }
    I++;
  }
  
  successful=dd_TRUE;

  return M;
}

static void cddinitGmp()
{
  static bool initialized;
  if(!initialized)
    {
      dd_set_global_constants();  /* First, this must be called. */
      initialized=true;
    }
}

bool LpSolverCddGmp::isFacet(const gbasis &g, gbasis::const_iterator i)
{
  bool ret;
  int index;
  dd_MatrixPtr M=NULL,M2=NULL,M3=NULL;
  dd_colrange d;
  dd_ErrorType err=dd_NoError;
  dd_rowset redrows,linrows,ignoredrows, basisrows;
  dd_colset ignoredcols, basiscols;
  //  long rank;
  mytype val;
  dd_DataFileType inputfile;
  FILE *reading=NULL;


  cddinitGmp();


  //  dd_init(val);

  M=vectorList2MatrixGmp(g, &err);

  if (err!=dd_NoError) goto _L99;

  d=M->colsize;

  //  redrows=dd_RedundantRows(M, &err);
  //  set_fwrite(stderr, redrows);


  index=0;
  for(gbasis::const_iterator J=g.begin();J!=g.end()&&J!=i;J++){index++;}
  if(index==g.size())assert(0);
  

  static dd_Arow temp;

  dd_InitializeArow(i->size(),&temp);

  ret= !dd_Redundant(M,index+1,temp,&err);

  
  dd_FreeMatrix(M);
  dd_FreeArow(i->size(),temp);

  if (err!=dd_NoError) goto _L99;
  return ret;
 _L99:
  assert(0);
  return false;

}

static LpSolverCddGmp theLpSolverCddGmp;
