#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <vector>
#include <list>

using namespace std;

#include "vektor.h"
#include "print.h"
#include "division.h"
#include "buchberger.h"
#include "toric.h"
#include "flip.h"
#include "lp.h"
#include "parser.h"
#include "radical.h"
#include "graver.h"
#include "fiber.h"
#include "decompose.h"
#include "macaulay2.h"
#include "normaliz.h"
#include "facet.h"
#include "walk.h"
#include "latticeideal.h"

#include "ep_latex.h"
#include "ep_tigers.h"

#include "ea_exhaustive.h"
#include "ea_reverse.h"
#include "ea_exhaustiveradical.h"
#include "ea_reverseradical.h"
#include "ea_hybrid.h"

#include "result.h"

#include "testresult.h"
#include "counter.h"

#include "application.h"


int option_latex;
int option_facetsuperset;
int option_facetsubset;
int option_facetsoplex;
int option_detailedprinting;

static char *changeFileExt(const char *name, const char *extension)
{
  int dotpos=-1;
  int i=0;

  while(name[i])
    {
      if(name[i]=='.')dotpos=i;
      i++;
    }
  if(dotpos==-1)
    dotpos=i;

  char *ret=(char*)malloc(strlen(extension)+dotpos+4);
  assert(ret);
  for(i=0;i<dotpos;i++)ret[i]=name[i];
  sprintf(ret+dotpos,"%s",extension);

  return ret;
}

static Timer globalTimer("Global-timer",1);

class Cats : Application
{
  const char *weightFilename;
  IntegerOption optionTraversalAlgorithm;
  ZeroOneOption optionLiftingAlgorithm;
  SimpleOption optionSecondIdeal;
  SimpleOption optionFileExtensions;
  StringOption optionVertexPrinting;
  //  StringOption optionCommentsFilename;
  StringOption optionResultsFilename;
  StringOption optionListFilename;
  StringOption optionInputFilename;
  StringOption optionWeightFilename;
  SimpleOption optionLatticeIdeal;
  StringOption optionLpSolver;
  SimpleOption optionPerformanceTimer;
  SimpleOption optionRadicalSplit;
  const char *helpText()
  {
    return 0;
  }
  void printHelp()
  {
    char *h=
      "This is CaTS version 2.2 (November 2003), a program for\n"
      "Computing Toric State polytopes by Anders Nedergaard Jensen.\n"
      "Homepage: http://www.soopadoopa.dk/anders/cats/cats.html\n"
      "\n"
      "CaTS computes all reduced Gr\\\"obner bases of a lattice ideal.\n"
      "It uses Birkett Huber's implementsion of the simplex algorithm\n"
      "or the libraries cddlib and SoPlex when linked properly.\n"
      "Read the documentation in the doc/ directory of the package.\n"
      "\n"
      "Links:\n"
      "cddlib: Komei Fukuda, \"cddlib version 0.93a\"\n"
      " http://www.cs.mcgill.ca/~fukuda/soft/cdd_home/cdd.html\n"
      "SoPlex: Roland Wunderling, \"Paralleler und Objekt-orientierter\n"
      " Simplex-Algorithmus\" ZIB technical report TR 96-09, Berlin 1996\n"
      " http://www.zib.de/Optimization/Software/Soplex/soplex.php\n"
      "TiGERS: Birkett Huber and Rekha R. Thomas \"Computing Gr\\\"obner\n"
      " Fans of Toric Ideals\", Experimental Mathematics, Vol 9 (2000),\n"
      " No. 3: 321-331\n"
      " http://www.math.washington.edu/~thomas/tigers.html\n"
      "\n"
      "OPTIONS:\n"
      "\n"
      "Traversal algorithm:\n"
      //      "-t0  None\n"
      "-t1  Exhaustive Search\n"
      "-t2 *Reverse Search\n"
      "-t3  Oriented Exhaustive Search\n"
      "-t4  Hybrid\n"
      "-t5  Exhaustive Search, fixed radical\n"
      "-t6  (Reverse Search, fixed radical) disabled in current version\n"
      "-t7  A-Graded Exhaustive Search\n"
      "-t8  A-Graded fixed radical exhaustive Search\n"
      "-w   Reverse search on each radical specified in the weight file from TOPCOM\n"
      "\n"
      "Extra input:\n"
      "-A   To give a second ideal - If the first Gr\\\"obner Basis is not where you\n"
      "     want to start the enumeration with (typically used with A-Graded search)\n"
      "\n"
      "Monomial lifting in A-Graded Exhaustive Search:\n"
      "-g0 *Use Fiber Search for lifting\n"
      "-g1  Use Graver Basis generated by 4ti2 for lifting\n"
      "\n"
      "Linear programming library:\n"
      //      "-fts0  Disable subset test\n"
      //      "-fts1 *Enable subset test\n"
      //      "-ftS0  Disable superset test\n"
      //      "-ftS1 *Enable superset test\n"
      "-fsoplex  SoPlex\n"
      "-fcdd     cddlib\n"
      "-fcddgmp  cddlib with gmp for exact arithmetics\n"
      "-fhuber   Huber\'s implementation\n"
      "\n"
      // "LaTeX:\n"
      // "-l0 Disable LaTeX output\n"
      // "-l1 Enable LaTeX output\n"
      // "\n"
      "Vertex printing:\n"
      //      "-pd Detailed printing of the graph (only on small examples and\n    together with -t1)\n"
      "-p0  No vertex printing\n"
      "-p1  Print the facet binomials for each cone in TiGERS' format when found\n"
      "-p1+ Print each reduced Gr\\\"obner basis stripped when found\n"
      "-p2  Print each monomial initial ideal in TiGERS' format when found\n"
      "-p2+ Print each monomial initial ideal stripped when found\n"
      //      "-p3  Print each reduced Gr\\\"obner basis in native format when found\n"
      "-qs  Split printing, print radical by radical, together with -p2+ and -w only\n"
      "\n"
      "Output file name options:\n"
      //      "-oc FILENAME  file for the comments (including the LaTeX \n              and .dvi files) Default: output_comments\n"
      "-or FILENAME  file for the results (time, size of graph, \n              floating point status ....) Default: output_results\n"
      "-ol FILENAME  file for listing the graph during traversal\n              (see options -p1, -p2 and -p3) Default: output_list\n"
      "\n"
      "Input file name options:\n"
      "-i FILENAME   input file\n"
      "-w FILENAME   weight file, containing heights for each radical\n"
      "\n"
      "File name options:\n"
      "-e  Force use of file name extensions (.heights,.results,.list)\n"
      "\n"
      "Lattice ideal options:\n"
      "-L  The ideal considered should be the lattice ideal of the lattice\n"
      "    generated by the rows of the input matrix rather than the toric\n"
      "    ideal of the input matrix\n"
      "\n"
      "Performance options:\n"
      "-T  Print internal timers when done\n\n"
      "Help:\n"
      "-h Print this help\n"
      "\n";
    fprintf(stdout,h);
    LpSolver::printList(stderr);
  }

  void useRadicalFile(gbasis g, Result *finalResult, EnumerationPrinter *printer)
  {
    bool radicalSplit=optionRadicalSplit.getValue();
    int i=0;
    int totalNumber=0;
    
    if(printer)printer->beginEnumeration(gbasis());
    
    assert(weightFilename);
    RadicalParser parser(weightFilename);
    while(parser.parseRadical())
      {
	//set weight vector
	vektor_i::weightenable=true;
	vektor_i::weight_double_enable=true;
	vektor_i::weight_double=parser.getWeightVector();
	assert(vektor_i::weight_double.size()==g.begin()->size());
	//compute fixed radical
	buchberger(g);
	gbasisminimal(&g);
	gbasisautoreduction(&g);	  
	
	if(radicalSplit)
	  {
	    printer->beginRadical(initialradical(g),parser.getNumberOfSymmetries());
	  }
	Result result;
	ReverseSearchFixedRadical algorithm;
	algorithm.setResult(&result);
	algorithm.setEnumerationTarget(printer);
	algorithm.enumerate(g);
	if(radicalSplit)
	  {
	    printer->endRadical();
	  }
	fprintf(stderr,"Radical no. %i: %i vertices, %i symmetries\n",i,result.numberOfVertices.get(),parser.getNumberOfSymmetries());
	
	totalNumber+=result.numberOfVertices.get()*parser.getNumberOfSymmetries();
	
	print(vektor_i::weight_double,stderr);
	fprintf(stderr,"Total number of initial ideals so far: %i\n",totalNumber); 
	i++;
      }
    if(printer)printer->endEnumeration();
    
    finalResult->numberOfVertices.assign(totalNumber);
  }
  



public:
  Cats():
    optionTraversalAlgorithm("-t",2,1,8),
    optionLiftingAlgorithm("-g",0),
    optionSecondIdeal("-A"),
    optionFileExtensions("-e"),
    optionVertexPrinting("-p","0"),
    //    optionCommentsFilename("-oc","output_comments"),
    optionResultsFilename("-or","output_results"),
    optionListFilename("-ol","output_list"),
    optionInputFilename("-i",0),
    optionWeightFilename("-w",0),
    optionLatticeIdeal("-L"),
    optionLpSolver("-f",""),
    optionPerformanceTimer("-T"),
    optionRadicalSplit("-qs")
  {
    registerOptions();
  }

  int main()
  {
    globalTimer.on();


    option_facetsuperset=1;
    option_facetsubset=1;
    option_latex=0;
    option_detailedprinting=0;

    setSupersetEnabled(option_facetsuperset);
    setSubsetEnabled(option_facetsubset);

    //    setSoplexEnabled(true);
    if(string(optionLpSolver.getValue())=="")
      lpSetSolver("SoPlex");
    else
      {
	bool ok=true;
	if(string(optionLpSolver.getValue())=="soplex")
	  ok=lpSetSolver("SoPlex");
	else if(string(optionLpSolver.getValue())=="huber")
	  ok=lpSetSolver("Huber's");
	else if(string(optionLpSolver.getValue())=="cdd")
	  ok=lpSetSolver("cdd");
	else if(string(optionLpSolver.getValue())=="cddgmp")
	  ok=lpSetSolver("cddgmp");
	else
	  {
	    fprintf(stderr,"Unknown Lp-Solver!\n");
	    assert(0);
	  }
	if(!ok)
	  {
	    fprintf(stderr,"Lp-Solver not linked!\n");
	    assert(0);
	  }
      }
    
  
    const char *inputFilename=optionInputFilename.getValue();
    weightFilename=optionWeightFilename.getValue();
    //    const char *commentsFilename=optionCommentsFilename.getValue();
    const char *listFilename=optionListFilename.getValue();
    const char *resultsFilename=optionResultsFilename.getValue();


    if(weightFilename==0 || weightFilename[0]==0)
      if(inputFilename)
	weightFilename=changeFileExt(inputFilename,".heights");
    if(inputFilename)
      {
	if(optionFileExtensions.getValue())
	  {
            //	    commentsFilename=changeFileExt(inputFilename,".comments");
	    resultsFilename=changeFileExt(inputFilename,".results");
	    listFilename=changeFileExt(inputFilename,".list");
	    weightFilename=changeFileExt(inputFilename,".heights");
	  }
	
	stdin=freopen(inputFilename,"r",stdin);
	assert(stdin);
      }
    
    fprintf(stderr,"Using files:\n");
    if(inputFilename)fprintf(stderr,"%s\n",inputFilename);
    //   if(commentsFilename)fprintf(stderr,"%s\n",commentsFilename);
    if(resultsFilename)fprintf(stderr,"%s\n",resultsFilename);
    if(weightFilename)fprintf(stderr,"%s\n",weightFilename);
    fprintf(stderr,"\n");
    
    EnumerationFilePrinter *printer=0;
    if(option_latex)
      {
	printer= new LatexEnumerationPrinter();
      }
    else
      {
	const char *s=optionVertexPrinting.getValue();
	//	fprintf(stderr,"s=%s\n",s);
	if(s==0 || string(s)==string("0") )
	  printer=0;
	else
	  {
	    TigersEnumerationPrinter *printer_ = new TigersEnumerationPrinter();
	    printer=printer_;
	    if(string(s)=="1")
	      {
		printer_->initialIdealPrintingSetEnabled(false);
		printer_->stripPrintingSetEnabled(false);
	      }
	    else if(string(s)=="2")
	      {
		printer_->initialIdealPrintingSetEnabled(true);
		printer_->stripPrintingSetEnabled(false);
	      }
	    else if(string(s)=="1+")
	      {
		printer_->initialIdealPrintingSetEnabled(false);
		printer_->stripPrintingSetEnabled(true);
	      }
	    else if(string(s)=="2+")
	      {
		printer_->initialIdealPrintingSetEnabled(true);
		printer_->stripPrintingSetEnabled(true);
	      }
	    else
	      {
		fprintf(stderr,"no printing option specified - aborting\n");
		exit(1);
	      }
	    printer_->open(string(listFilename));
	  }
      }
    
    if(option_latex)
      {
	static char buf[256];
        //	sprintf(buf,"%s.tex",commentsFilename);
	print_beginLaTeX(buf);
      }
    else
      {
        //	stdout=freopen(commentsFilename,"w",stdout);
      }

    print_alfabeta();
    
    basis_i A;
    gbasis inputgbasis;
    int inputtype=FileParser(stdin).parseInput(&A,&inputgbasis);
    print_timebegin();
    
    gbasis g;
    
    Result result;
    
    gbasis secondIdeal;
    
    
    switch(inputtype)
      {
      case 1://matrix
	if(optionLatticeIdeal.getValue())
	  {
	    g=latticeIdeal(A);
	    AsciiPrinter(stderr).printGroebnerBasis(g);
	  }
	else
	  {
	    g=toric1(A);
	  }
	printf("\n");
	result.fillOutBasicFields(A);
	break;
      case 2://gbasis
	g=inputgbasis;
	gbasisminimal(&g);
	gbasisautoreduction(&g);
	if(optionTraversalAlgorithm.getValue()!=1 && optionTraversalAlgorithm.getValue()!=5)
	  {
	    if(optionSecondIdeal.getValue())
	    {
	      fprintf(stderr,"ERROR -A not supported");
	      assert(0);
	    }
            //	    const char *comment;
	    if(optionTraversalAlgorithm.getValue()==6)
	      {
		assert(0);//Change term order
	      }
	    else
	    {
	    }
            fprintf(stderr,"\n");
            AsciiPrinter(stderr).printGroebnerBasis(g);
	  }
      break;
      }
    
    if(optionSecondIdeal.getValue())
      {
	fprintf(stderr,"Type in fake Gr\\\"obner basis for second ideal:");
	secondIdeal=FileParser(stdin).parseBinomialList(g.begin()->size());
      }
    
    fflush(stdout);
    
    if(!optionWeightFilename.getValue())
      {
	EnumerationAlgorithm *algorithm=0;
	switch(optionTraversalAlgorithm.getValue())
	  {
	  case 1:
	    {
	      printf("\n\nComputing the Gr\\\"obner fan using exhaustive search...\n\n");
	      ExhaustiveSearch *algorithm_= new ExhaustiveSearch();
	      algorithm=algorithm_;
	      algorithm_->setUseOrdered(false);
	    }
	    break;
	  case 2:
	    printf("\n\nComputing the Gr\\\"obner fan using the reverse search method...\n\n");
	    algorithm= new ReverseSearch();
	    break;
	  case 3:
	    {
	      printf("\n\nComputing the Gr\\\"obner fan using a faster exhaustive search...\n\n");
	      ExhaustiveSearch *algorithm_= new ExhaustiveSearch();
	      algorithm=algorithm_;
	      algorithm_->setUseOrdered(false);
	    }
	    break;
	  case 4:
	    printf("\n\nComputing the Gr\\\"obner fan using a hybrid algorithm...\n\n");
	    algorithm= new Hybrid();
	    break;
	  case 5:
	    printf("\n\nExhaustive, fixed radical algorithm...\n\n");
	    algorithm= new ExhaustiveSearchFixedRadical();
	    break;
	  case 6:
	    printf("\n\nReverse, fixed radical algorithm...\n\n");
	    algorithm= new ReverseSearchFixedRadical();
	    break;
	  case 7:
	  case 8:
	    if(optionTraversalAlgorithm.getValue()==7)
	      printf("\n\nExhaustive Search for connected A-Graded monomial ideals...\n\n");
	    else
	      printf("\n\nExhaustive Search for connected A-Graded monomial ideals - fixed Radical...\n\n");
	    
	    if(optionLiftingAlgorithm.getValue())
	      {
		if(inputtype!=1)
		  {
		    fprintf(stderr,"No A-Matrix given\n");
		  }
		algorithm= (optionTraversalAlgorithm.getValue()==7)?
		  new ExhaustiveSearchAGraded(A):
		  new ExhaustiveSearchAGradedFixedRadical(A);
	      }
	    else
	      {
		if(inputtype!=1)
		  {
		    fprintf(stderr,"Assuming the first input was a Gr\\\"obner basis!!!\n");
		    algorithm= (optionTraversalAlgorithm.getValue()==7)?
		      new ExhaustiveSearchAGraded(g):
		      new ExhaustiveSearchAGradedFixedRadical(g);
		  }
		else
		  {
		    algorithm= (optionTraversalAlgorithm.getValue()==7)?
		      new ExhaustiveSearchAGraded(toric1(A)):
		      new ExhaustiveSearchAGradedFixedRadical(toric1(A));
		  }
	      }
	    break;
	  default:
	    assert(0);
	    break;
	  }
	
	assert(algorithm);
	
	if(printer)
	  algorithm->setEnumerationTarget(printer);

	algorithm->setResult(&result);
	
	fprintf(stderr,"enumeration begin\n");
	if(optionSecondIdeal.getValue())
	  algorithm->enumerate(secondIdeal);
	else
	  algorithm->enumerate(g);
	fprintf(stderr,"enumeration end\n");
        if(result.numberOfEdges.get()==-1)
          printf("\n\nNumber of enumerated vertices: %i.\n\n",result.numberOfVertices.get());  
        else
          printf("\n\nNumber of enumerated vertices: %i, with %i edges.\n\n",result.numberOfVertices.get(),result.numberOfEdges.get());  
	result.print(cerr);
	
	delete algorithm;
      }
    else
      {
	useRadicalFile(g,&result,printer);
      }
    //  fprintf(stderr,"TO DO: write the time and the floating point status to results too\n");
    //  fprintf(stderr,"TO DO: write the results as an xml file\n");

    result.save(string(resultsFilename));
    
    
    if(option_latex)
      {
	print_endLaTeX();
    }
    else
      {
	fclose(stdout);
      }
    
    if(printer)printer->close();
    
    /*    if(lp_status==0)
      {
	fprintf(stderr,"No floating point problems detected.\n");
      }
    else
      {
	fprintf(stderr,"Floating point troubles! Result is not reliable.\n");
      }
    */
    globalTimer.off();
    if(optionPerformanceTimer.getValue())Timer::printList();

    return 0;
  }
};


static Cats theApplication;


