#include "normaliz.h"

#include <map>
#include <string>

#include "flip.h" //definition of gfan
#include "printer.h"
#include "parser.h"
#include "programs.h"

FILE *callNormaliz(gbasis const &g)
{
  int d;
  assert(g.size());

  FILE *f=fopen("normalizdata.in","w");
  assert(f);
  fprintf(f,"%i\n%i\n",g.size(),d=g.begin()->size());

  for(gbasis::const_iterator i=g.begin();i!=g.end();i++)
    {
      for(int j=0;j!=i->size();j++)
	{
	  if(j!=0)fprintf(f," ");
	  fprintf(f,"%i",(*i)[j]);
	}
      fprintf(f,"\n");
    }

  fprintf(f,"0\n");
  fclose(f);

  //  system("~/math/normalizExamples/normaliz/source/normaliz normalizdata");
  system((string(programNameNormaliz)+string(" normalizdata")).c_str());

  f=fopen("normalizdata.out","r");
  assert(f);

  return f;
}


gbasis normaliz(gbasis const &g)
{
  FILE *f=callNormaliz(g);

  int d=g.begin()->size();
  int n;
  char temp[1024];
  fscanf(f,"%i\n",&n);
  fgets(temp,1024,f);

  gbasis b;
  for(int i=0;i<n;i++)
    {
      vektor_i v(d);
      for(int j=0;j<d;j++)
	{
	  int a;
	  fscanf(f,"%i ",&a);
	  v[j]=a;
	}
      b.push_back(v);
    }

  fclose(f);

  return b;
}


//--------------------------------------------------
// isNormal - using normaliz
//--------------------------------------------------
// positive span of input should be a pointed cone!!!
bool isNormal(gbasis const &g)
{
  gbasis hilbertBasis=normaliz(g);

  for(gbasis::const_iterator h=hilbertBasis.begin();h!=hilbertBasis.end();h++)
    {
      bool found=false;
      for(gbasis::const_iterator v=g.begin();v!=g.end();v++)
	if(*v==*h){found=true;break;}
      if(!found)return false;
    }
  return true;
}

//--------------------------------------------------
// Delta-Normal test using TOPCOM and normaliz
//--------------------------------------------------

gfan points2triangs(basis_i const &A, bool regular)
{
  gfan ret;

  FILE *f=fopen("TOPCOMdata.dat","w");

  assert(f);
  TopcomPrinter(f).printVectorList(toLinkedMatrix(A));
  fclose(f);
  if(regular)
    system((string(programNameTopcomPoints2Triangs)+string(" --regular <TOPCOMdata.dat >TOPCOMdata.tri")).c_str());
  else
    system((string(programNameTopcomPoints2Triangs)+string(" <TOPCOMdata.dat >TOPCOMdata.tri")).c_str());
  
  f=fopen("TOPCOMdata.tri","r");
  assert(f);
  while(1)
    {
      int c;
      do
	{
	  c=fgetc(f);
	}
      while(c!=EOF && c!='=');
      if(c==EOF)break;
      ret.push_back(FileParser(f).parseMatrix());
    }
  fclose(f);

  if(ret.size()==0)fprintf(stderr,"Error with TOPCOM? - did you transpose the input?\n");

  return ret;
}

gfan regularTriangulations(basis_i const &A)
{
  return points2triangs(A, true);
}

gfan connectedToRegularTriangulations(basis_i const &A)
{
  return points2triangs(A, false);
}

gbasis computeWalls(gbasis const &A)
{
  gbasis ret;
  FILE *f=callNormaliz(A);
  
  {
    char temp[1024];
    while(1)
      {
	fgets(temp,1024,f);
	bool containsY=false;
	int i=0;
	while(temp[i]){
	  if(temp[i]=='y')containsY=true;
	  i++;
	}
	if(containsY)break;
      }
    int nWalls=0;
    sscanf(temp,"%i",&nWalls);
    //    assert(nWalls==triangle.size());
    for(int a=0;a<nWalls;a++)
      {
	vektor_i v(A.begin()->size());
	for(int b=0;b<v.size();b++)
	  {
	    int c;
	    fscanf(f,"%i\n",&c);
	    v[b]=c;
	  }
	ret.push_back(v);
      }
  }
  fclose(f);

  return ret;
}

gbasis vectorsInsideTriangle(basis_i const &A, vektor_i triangle)
{
  gbasis ret;
  gbasis simplex;

  for(int i=0;i<triangle.size();i++)
    simplex.push_back(A[triangle[i]]);

  gbasis walls=computeWalls(simplex);

  for(int i=0;i<A.size();i++)
    {
      bool inside=true;
      for(gbasis::iterator j=walls.begin();j!=walls.end();j++)
	{
	  if(dot(*j,A[i])<0)inside=false;
	}
      if(inside)
	ret.push_back(A[i]);
    }

  return ret;
}


class triangleResult
{
public:
		 vektor_i subset;
  triangleResult(vektor_i const &v):
    subset(v)
  {
  }
  bool operator<(class triangleResult const &b)const
  {
    for(int i=0;i<subset.size();i++)
      {
	if(subset[i]<b.subset[i])return true;
	if(subset[i]>b.subset[i])return false;
      }
    return false;
  }
};

typedef map<triangleResult,bool> triangleCache;


bool isDeltaNormal(basis_i const &A, gbasis *triangulation, bool useNotOnlyRegularTriangulations, int printComments)
{
  // if the answer is true with useNotOnlyRegularTriangulations=true the right answer could still be false since the triangulation could be non-regular

  triangleCache cache;
  gfan tris;
  if(useNotOnlyRegularTriangulations)
    tris=connectedToRegularTriangulations(A);
  else
    tris=regularTriangulations(A);
 
  if(printComments>0)
    {
      fprintf(stderr,"Number of regular triangulations: %i\n",tris.size());
    }
  int triangulationNumber=0;
  for(gfan::const_iterator delta=tris.begin();delta!=tris.end();delta++)
    {
      if(printComments>0)
	{
	  if(printComments>1 || (triangulationNumber&255)==0)
	  fprintf(stderr,"Testing triangulation %i. Number of triangles in triangulation:%i\n",triangulationNumber,delta->size());
	}
      if(printComments>1)
	{
	  fprintf(stderr,"Triangulation:\n");
	  AsciiPrinter(stderr).printVectorList(*delta);fprintf(stderr,"\n");
	}
      bool deltaNormal=true;
      for(gbasis::const_iterator triangle=delta->begin();triangle!=delta->end();triangle++)
	{
	  if(cache.count(triangleResult(*triangle)))
	    {
	      if(!cache[triangleResult(*triangle)]){deltaNormal=false;break;}
	    }
	  else
	    {
	      gbasis vectors=vectorsInsideTriangle(A,*triangle); 
	      
	      if(printComments>1)
		{
		  fprintf(stderr,"Testing triangle:\n");
		  AsciiPrinter(stderr).printVector(*triangle);
		  fprintf(stderr,"\nPoints inside triangle:\n");
		  AsciiPrinter(stderr).printVectorList(vectors);
		  fprintf(stderr,"\n");
		}
	      gbasis hilbertBasis=normaliz(vectors);
	      if(printComments>1)
		{
		  fprintf(stderr,"Hilbert basis:\n");
		  AsciiPrinter(stderr).printVectorList(hilbertBasis);
		  fprintf(stderr,"\n");
		}
	      
	      bool triangleIsNormal=true;
	      for(gbasis::const_iterator h=hilbertBasis.begin();h!=hilbertBasis.end();h++)
		{
		  bool found=false;
		  for(gbasis::const_iterator v=vectors.begin();v!=vectors.end();v++)
		    if(*v==*h){found=true;break;}
		  if(!found){  triangleIsNormal=false;break;}
		}
	      if(printComments>1)
		{
		  if(triangleIsNormal)
		    fprintf(stderr,"The subset of vectors inside this triangle is normal.\n");
		  else
		    fprintf(stderr,"The subset of vectors inside this triangle is not normal, so the input point configuration is not delta normal with respect to the current triangulation.\n\n");
		}
	      cache[triangleResult(*triangle)]=triangleIsNormal;
	      if(!triangleIsNormal){deltaNormal=false;break;}
	    }
	  if(!deltaNormal)break;
	}
      if(deltaNormal)
	{
	  if(printComments>1)
	    {
	      fprintf(stderr,"Configuration is delta normal with respect to this triangulation:\n");
	      AsciiPrinter(stderr).printVectorList(*delta);
	      fprintf(stderr,"\n");
	    }
	  if(triangulation)*triangulation=*delta;
	  return true;
	}
      triangulationNumber++;
    }
  if(printComments>1)
    {
      fprintf(stderr,"The configuration was not delta normal with respect to any regular triangulation.\n");
    }
  return false;
}


void printConvexHullInfo(const basis_i &A)
{
  FILE *f=stderr;
  gbasis a=toLinkedMatrix(A);
  fprintf(stderr,"TESTingA\n");
  fprintf(f,"TESTing\n");
  gbasis walls=computeWalls(a);
  

  for(gbasis::const_iterator i=walls.begin();i!=walls.end();i++)
    {
      AsciiPrinter(f).printVector(*i);
      fprintf(f,"\n                               ");
      int num=0;
      for(gbasis::const_iterator j=a.begin();j!=a.end();j++)
	{
	  if(dot(*j,*i)==0)
	    fprintf(f," %c",'a'+num);
	  else if(dot(*j,*i)>0)
	    fprintf(f,"  ");
	  else
	    fprintf(f,"error\n");
	  num++;
	}
      fprintf(f,"\n");
    }
}
