#include "parser.h"
#include <stdio.h>
#include <assert.h>
#include "print.h"
#include "printer.h"

/*
To do:
The asserts in this should be replaced by asserts that are always checked even
when we are not in debug mode.
*/

//--------------------------------------------------------
// Radical File Parser
//--------------------------------------------------------

RadicalParser::RadicalParser(const char *filename)
{
  f=fopen(filename,"r");
  assert(f);
  mode=1;
}


RadicalParser::~RadicalParser()
{
  fclose(f);
  f=0;
}

bool RadicalParser::readWeightVector()
{
  char line[4096];
  int c;
  while((c=fgetc(f))!='(' && c!='T' && c!=EOF);
  ungetc(c,f);
  if(c!='(')return false;
  fgets(line,4096,f);
  vektor_f weight_vector=StringParser(line).parseFloatingPointVector();
  //  vektor_f weight_vector(line);
  currentWeightVector=weight_vector;
  return true;
}

bool RadicalParser::parseRadical()
{
  if(!readWeightVector())return false;
  if(mode==0)
    {
      currentNumberOfSymmetries=1;
      return true;
    }

  currentNumberOfSymmetries=1;
  while(readWeightVector())currentNumberOfSymmetries++;

  int c;
  while((c=fgetc(f))!='(' && c!='T' && c!=EOF);

  assert(c=='T');
  while((c=fgetc(f))!='\n' && c!=EOF);

  assert(c=='\n');
  return true;
}


vektor<double> RadicalParser::getWeightVector()
{
  return currentWeightVector;
}


int RadicalParser::getNumberOfSymmetries()
{
  return currentNumberOfSymmetries;
}


//--------------------------------------------------
// Parser
//--------------------------------------------------


//--------------------------------------------------
// CharacterBasedParser
//--------------------------------------------------

static void changeVectorLengths(gbasis *g)
{
  int n=0;
  for(gbasis::const_iterator i=g->begin();i!=g->end();i++)
    if(i->size()>n)n=i->size();
  for(gbasis::iterator i=g->begin();i!=g->end();i++) /* Let's hope that noone tries to compile this with Visual C */
    i->resize(n);
}


int CharacterBasedParser::nextNonBlank()
{
 int c=getChar();

 while(c!=0 && (c=='\t' || c==' ' || c=='\n'))c=getChar();

 return c;
}


bool CharacterBasedParser::isLeftBrackets(int c)
{
  return c=='(' || c=='[' || c=='{';
}


bool CharacterBasedParser::isRightBrackets(int c)
{
  return c==')' || c==']' || c=='}';
}


int CharacterBasedParser::parseInt()
{
  int ret=0;
  int c=getChar();


  while(c!=0 && (c=='\t' || c==' ' || c=='\n'))c=getChar();
  if(c=='-')return -parseInt();
  while(c!=0)
    {
      if(c>='0' && c<='9')
	{
	  ret=ret*10+(c-'0');
	  c=getChar();
	}
      else
	break;
    }
  ungetChar(c);

  return ret;
}


double CharacterBasedParser::parseFloat()
{
  double ret=0;
  static char s[64];
  int i=0;

  int c=nextNonBlank();
  while(i<64 && ((c>='0' && c<='9') || c=='.' || c=='-') )
    {
      s[i++]=c;
      c=getChar();
    }      
  assert(i<64);
  s[i]=0;

  ungetChar(c);

  sscanf(s,"%f",&ret); //does this work? Are we reading floats or doubles?

  return ret;
}


bool CharacterBasedParser::isVariable(int c)
{
  return (c>='a' && c<='z') || (c>='A' && c<='Z');
}


int CharacterBasedParser::variableIndex(int c)
{
  if(c>='a' && c<='z')return c-'a';
  if(c>='A' && c<='Z')return c-'A'+26;
  assert(0);
  return 0;
}


vektor_i CharacterBasedParser::parseMonomial(int dimension)
{
  /* Parses a monomial without spaces and without sign */

  vektor_i ret((dimension==-1)?0:dimension);

  int c=nextNonBlank();
  //  int c=getChar();

  if(c=='1')return ret;

  while(c!=0 && (c=='\t' || c==' ' || c=='\n'))c=getChar();
  while(c!=0)
    {
      if(isVariable(c))
	{
	  int index=variableIndex(c);
	  int exponent=1;
	  if(dimension==-1)
	    ret.grow(index+1);
	  else
	    assert(index<dimension);
	  c=getChar();
	  if(c=='^')
	    {
	      exponent=parseInt();
	      c=getChar();
	    }
	  ret[index]+=exponent;
	}
      else if(c=='*')
	{
	  c=getChar();
	}
      else break;
    }
  ungetChar(c);

  return ret;
}


gbasis CharacterBasedParser::parseMonomialList(int dimension)
{
  gbasis ret;
  int c=nextNonBlank();

  assert(isLeftBrackets(c));
  do
    ret.push_back(parseMonomial(dimension));
  while((c=nextNonBlank())==',');
  assert(isRightBrackets(c));

  if(dimension==-1)changeVectorLengths(&ret);

  return ret;
}


gbasis CharacterBasedParser::parseBinomialList(int dimension)
{
  gbasis ret;
  int c=nextNonBlank();

  assert(isLeftBrackets(c));
  do
    ret.push_back(parseBinomial(dimension));
  while((c=nextNonBlank())==',');
  assert(isRightBrackets(c));

  if(dimension==-1)changeVectorLengths(&ret);

  return ret;
}


/*gfan CharacterBasedParser::parseMonomialListList(int dimension)
{
  gfan ret;
  int c=nextNonBlank();

  assert(isLeftBrackets(c));
  do
    ret.push_back(parseMonomialList(dimension));
  while((c=nextNonBlank())==',');
  assert(isRightBrackets(c));

  return ret;
}
*/

vektor_i CharacterBasedParser::parseBinomial(int dimension)
{
  vektor_i lt,tl;

  lt=parseMonomial(dimension);
  int c=nextNonBlank();

  assert(c=='-');
  tl=parseMonomial(dimension);

  if(dimension==-1)
    {
      int n=lt.size();
      int N=tl.size();
      if(N>n)n=N;
      lt.grow(n);
      tl.grow(n);
    }
  //  fprintf(stderr,"parsed binomial:");
  //  AsciiPrinter(stderr).printBinomial(lt-tl);
  //  fprintf(stderr,"\n");

  return lt-tl;
}


vektor_i CharacterBasedParser::parseIntegerVector()
{
  list<int> temp;
  int c=nextNonBlank();

  assert(isLeftBrackets(c));
  do
    temp.push_back(parseInt());
  while((c=nextNonBlank())==',');
  assert(isRightBrackets(c));

  vektor_i ret(temp.size());

  {
    int j=0;
    for(list<int>::iterator i=temp.begin();i!=temp.end();i++)
      ret[j++]=*i;
  }
  
  return ret;
}


vektor_f CharacterBasedParser::parseFloatingPointVector()
{
  list<double> temp;
  int c=nextNonBlank();

  assert(isLeftBrackets(c));
  do
    temp.push_back(parseFloat());
  while((c=nextNonBlank())==',');
  assert(isRightBrackets(c));

  vektor_f ret(temp.size());

  {
    int j=0;
    for(list<double>::iterator i=temp.begin();i!=temp.end();i++)
      ret[j++]=*i;
  }
  
  return ret;
}


gbasis CharacterBasedParser::parseMatrix()
{
  gbasis ret;
  int c=nextNonBlank();

  switch(c)
    {
    case 'M':
      {
	c=nextNonBlank();
	assert(c==':');
	c=nextNonBlank();
	assert(isLeftBrackets(c));
	
	int nrows=parseInt();
	int ncolumns=parseInt();
	
	c=nextNonBlank();
	assert(c==':');

	for(int i=0;i<ncolumns;i++)
	  {
	    vektor_i v(nrows);
	    ret.push_back(v);
	  }
	for(int j=0;j<nrows;j++)
	  for(gbasis::iterator k=ret.begin();k!=ret.end();k++)
	    {
	      k->v[j]=parseInt();
	    }
	c=nextNonBlank();
	assert(isRightBrackets(c));
      }
      break;
    default:
      {
	assert(isLeftBrackets(c));
	while(!isRightBrackets(c=nextNonBlank()))
	  {
	    ungetChar(c);
	    ret.push_back(parseIntegerVector());
	    c=nextNonBlank();
	    if(c!=',')ungetChar(c);
	  }
      }
      break;
    }

  return ret;
}


StandardPair CharacterBasedParser::parseStandardPair(int dimension)
{
  assert(dimension!=-1);
  StandardPair s;
  int c=nextNonBlank();

  assert(isLeftBrackets(c));
  s.root=parseMonomial(dimension);
  c=nextNonBlank();
  assert(c==',');
  
  gbasis g=parseMonomialList(dimension);
  s.face=vektor_i(dimension);
  for(gbasis::iterator i=g.begin();i!=g.end();i++)
    s.face+=*i;//should be ok - input should be minimal

  c=nextNonBlank();
  assert(isRightBrackets(c));

  return s;
}


StandardPairList CharacterBasedParser::parseStandardPairList(int dimension)
{
  assert(dimension!=-1);
  StandardPairList ret;
  int c=nextNonBlank();

  //  fprintf(stderr,"c=\'%c\',%i\n",c,c);

  assert(isLeftBrackets(c));
  do
    ret.push_back(parseStandardPair(dimension));
  while((c=nextNonBlank())==',');
  assert(isRightBrackets(c));

  return ret;
}


int CharacterBasedParser::parseInput(basis_i *matrix_, gbasis *basis)
{
  basis_i B;
  int c;

  fprintf(stderr,"Type in matrix or Groebner basis:");
    {
      int c=nextNonBlank();
      switch(c)
	{
	case 'R':
	  {
	    gbasis g;
	    int dim;
	    c=nextNonBlank();
	    assert(c==':');
	    dim=parseInt();
	    c=nextNonBlank();
	    assert(c=='G');
	    c=nextNonBlank();
	    assert(c==':');
	    g=parseBinomialList(dim);

	    fprintf(stderr,"\nParsed Gr\\\"obner basis:\n");
	    print_tigers_strip(g,0,stderr);

	    if(basis)*basis=g;
	    return 2;
	  }
	  break;
	case '_':
	  c=nextNonBlank();
	  assert(c=='A');
	  c=nextNonBlank();
	  switch(c)
	    {
	    case '3':
	      B=toIndexedMatrix(StringParser("{(1)(2)(3)}").parseMatrix());
	      break;
	    case '4':
	      B=toIndexedMatrix(StringParser("{(1)(2)(3)(4)}").parseMatrix());
	      break;
	    case '5':
	      B=toIndexedMatrix(StringParser("{(1)(2)(3)(4)(5)}").parseMatrix());
	      break;
	    case '6':
	      B=toIndexedMatrix(StringParser("{(1)(2)(3)(4)(5)(6)}").parseMatrix());
	      break;
	    case '7':
	      B=toIndexedMatrix(StringParser("{(1)(2)(3)(4)(5)(6)(7)}").parseMatrix());
	      break;
	    case '8':
	      B=toIndexedMatrix(StringParser("{(1)(2)(3)(4)(5)(6)(7)(8)}").parseMatrix());
	      break;
	    case 'G':
	      B=toIndexedMatrix(StringParser("{(1)(2)(3)(7)(8)(9)}").parseMatrix());
	      break;
	    default:
	      fprintf(stderr,"unknown matrix name\n");
	      assert(0);
	      break;
	    }
	  break;
	default:
	  {
	    ungetChar(c);
	    B=toIndexedMatrix(parseMatrix());
	  }
	  break;
	}
    }
  fprintf(stderr,"\nInput Matrix Columns:\n");
  print(B,stderr);
  if(matrix_)*matrix_=B;

  return 1;
}


//--------------------------------------------------
// FileParser
//--------------------------------------------------

int FileParser::getChar()
{
  int c=getc(f);
  //  fprintf(stderr,"getChar c=\'%c\',%i\n",c,c);
  if(c==EOF)return 0;

  return c;
}


void FileParser::ungetChar(int c)
{
  ungetc(c,f);
}


FileParser::FileParser(FILE *f)
{
  this->f=f;
}


//--------------------------------------------------
// StringParser
//--------------------------------------------------

int StringParser::getChar()
{
  if(hasUngotten)
    {
      hasUngotten=false;
      return ungotten;
    }
  return s[index++];
}

void StringParser::ungetChar(int c)
{
  assert(hasUngotten==false);
  hasUngotten=true;
  ungotten=c;
}

StringParser::StringParser(char *s):
  index(0),
  hasUngotten(false)
{
  this->s=s;
}
