#ifndef PARSER_H
#define PARSER_H

#include "vektor.h"
#include "flip.h" // should not be included, but gfan is defined here!
#include "decompose.h" //standard pairs are defined here

class RadicalParser
{
  vektor<double> currentWeightVector;
  int currentNumberOfSymmetries;
  int mode;
  FILE *f;
  bool readWeightVector();
 public:
  RadicalParser(const char *filename);
  ~RadicalParser();
  bool parseRadical();
  vektor<double> getWeightVector();
  int getNumberOfSymmetries();
};


class Parser
{
 public:
  virtual int parseInt()=0;
  virtual vektor_i parseMonomial(int dimension)=0;
  virtual gbasis parseMonomialList(int dimension)=0;
  virtual vektor_i parseIntegerVector()=0;
};


class CharacterBasedParser : public Parser
{
 private:
  int nextNonBlank();
  bool isLeftBrackets(int c);
  bool isRightBrackets(int c);
  bool isVariable(int c);
  int variableIndex(int c);
 protected:
  virtual int getChar()=0;
  virtual void ungetChar(int c)=0;
 public:
  int parseInt();
  double parseFloat();
  vektor_i parseMonomial(int dimension=-1);
  gbasis parseMonomialList(int dimension=-1);
  gbasis parseBinomialList(int dimension=-1);
  gfan parseMonomialListList(int dimension=-1);
  vektor_i parseBinomial(int dimension=-1);
  vektor_i parseIntegerVector();
  vektor_f parseFloatingPointVector();
  gbasis parseMatrix();
  StandardPair parseStandardPair(int dimension=-1);
  StandardPairList parseStandardPairList(int dimension=-1);
  int parseInput(basis_i *matrix, gbasis *basis);
// returns 1 if the input was a matrix and writes the input matrix in matrix
// returns 2 if the input was a Gr\"obner basis and writes the input to basis
};


class FileParser : public CharacterBasedParser
{
  FILE *f;
 protected:
  virtual int getChar();
  virtual void ungetChar(int c);
 public:
  FileParser(FILE *f);
};


class StringParser : public CharacterBasedParser
{
  char *s;
  int index;
  bool hasUngotten;
  char ungotten;
 protected:
  virtual int getChar();
  virtual void ungetChar(int c);
 public:
  StringParser(char *s);
};

#endif
