#include "printer.h"

//--------------------------------------------------
// LatexPrinter
//--------------------------------------------------

string Printer::variableIndexToString(int i)
{
  static char s[4];
  if(i>=0 && i<26)sprintf(s,"%c",i+'a');
  else if(i>=26 && i<52)sprintf(s,"%c",i+'A'-26);
  else assert(0);
  return string(s);
}


//--------------------------------------------------
// LatexPrinter
//--------------------------------------------------

void LatexPrinter::pushMathMode()
{
  if(mathModeLevel==0)fprintf(f," $ ");
  mathModeLevel++;
}


void LatexPrinter::popMathMode()
{
  mathModeLevel--;
  if(mathModeLevel==0)fprintf(f," $ ");
}


void LatexPrinter::printMonomial(const vektor_i &m, int sign)
{
  pushMathMode();

  bool variablePrinted=false;
  for(int i=0;i<m.size();i++)if(m[i]*sign>0)
    {
      fprintf(f,"%s",variableIndexToString(i).c_str());
      //      fprintf(f,"%c",i+'a');
      if(int(m[i]*sign)!=1)
	{
	  fprintf(f,"^{%i}",int(m[i]*sign));
	}
      variablePrinted=true;
    }
  if(!variablePrinted)fprintf(f,"1");

  popMathMode();
}


void LatexPrinter::printBinomial(const vektor_i &b)
{
  pushMathMode();
  printMonomial(b,1);
  fprintf(f,"-");
  printMonomial(b,-1);
  popMathMode();
}


void LatexPrinter::printVector(const vektor_i &v)
{
  int s=v.size();
  int i=0;

  fprintf(f,"$(");
  if(i<s)fprintf(f,"%i",(v[i]));
  for(i++;i<s;i++)fprintf(f,",%i",(v[i]));
  fprintf(f,")$\n");
}


void LatexPrinter::printVectorList(const gbasis &g)
{
}


void LatexPrinter::printMatrix(const basis_i &m)
{
  int col=m.size();
  if(col==0)return;
  int row=m[0].size();
  
  fprintf(f,"$\\left( \\begin{array}{rrrrrrrrrrrrrrrrrrr}\n");
  for(int i=0;i<row;i++)
    {
      int j;
      for(j=0;j<col-1;j++)
	printf("%i&",m[j][i]);
      printf("%i \\\\\n",m[j][i]);
    }
  fprintf(f,"\\end{array}\\right )$\n");
}


void LatexPrinter::printTransposedMatrix(const basis_i &m)
{
  int row=m.size();
  if(row==0)return;
  int col=m[0].size();
  
  fprintf(f,"$\\left( \\begin{array}{rrrrrrrrrrrrrrrrrrr}\n");
  for(int i=0;i<row;i++)
    {
      int j;
      for(j=0;j<col-1;j++)
	printf("%i&",m[i][j]);
      printf("%i \\\\\n",m[i][j]);
    }
  fprintf(f,"\\end{array}\\right )$\n");
}


void LatexPrinter::printGroebnerBasis(const gbasis &g)
{
  pushMathMode();
  fprintf(f,"\\{ ");
  for(gbasis::const_iterator i=g.begin();i!=g.end();i++)
    {
      if(i!=g.begin())fprintf(f,", ");
      printBinomial(*i);
    }
  fprintf(f,"\\} ");
  popMathMode();
}


void LatexPrinter::printGroebnerBasisNewLine(const gbasis &g)
{
}


void LatexPrinter::printMonomialList(const gbasis &g)
{
}


void LatexPrinter::printMonomialListNewLine(const gbasis &g)
{
}


void LatexPrinter::printMonomialIdeal(const gbasis &g)
{
  pushMathMode();
  fprintf(f,"\\rangle ");
  for(gbasis::const_iterator i=g.begin();i!=g.end();i++)
    {
      if(i!=g.begin())fprintf(f,", ");
      printMonomial(*i,1);
    }
  fprintf(f,"\\langle ");
  popMathMode();
}


void LatexPrinter::printMonomialIdealList(const gfan &l)
{
  pushMathMode();
  fprintf(f,"{ ");
  for(gfan::const_iterator i=l.begin();i!=l.end();i++)
    {
      if(i!=l.begin())fprintf(f,", ");
      printMonomialIdeal(*i);
    }
  fprintf(f,"} ");
  popMathMode();
}


void LatexPrinter::printStandardPair(const StandardPair &g)
{
}


void LatexPrinter::printStandardPairList(const StandardPairList &g)
{
}


void LatexPrinter::printString(const string &s)
{
}


//--------------------------------------------------
// AsciiPrinter
//--------------------------------------------------

void AsciiPrinter::printMonomial(const vektor_i &m, int sign)
{
  bool variablePrinted=false;
  for(int i=0;i<m.size();i++)if(m[i]*sign>0)
    {
      if(variablePrinted)fprintf(f,"*");
      fprintf(f,"%s",variableIndexToString(i).c_str());
      //      fprintf(f,"%c",i+'a');
      if(int(m[i]*sign)!=1)
	{
	  fprintf(f,"^%i",int(m[i]*sign));
	}
      variablePrinted=true;
    }
  if(!variablePrinted)fprintf(f,"1");
}


void AsciiPrinter::printBinomial(const vektor_i &b)
{
  printMonomial(b,1);
  fprintf(f,"-");
  printMonomial(b,-1);
}


void AsciiPrinter::printVector(const vektor_i &v)
{
  fprintf(f,"%c",vectorLeftBrackets());
  for(int i=0;i<v.size();i++)
    {
      if(i!=0)fprintf(f,",");
      fprintf(f,"%i",v[i]);
    }
  fprintf(f,"%c",vectorRightBrackets());
}


void AsciiPrinter::printVectorList(const gbasis &g)
{
  fprintf(f,"%c",vectorListLeftBrackets());
  for(gbasis::const_iterator i=g.begin();i!=g.end();i++)
    {
      if(i!=g.begin())fprintf(f,",");
      fprintf(f,"\n");
      printVector(*i);
    }
  fprintf(f,"\n%c\n",vectorListRightBrackets());
}


void AsciiPrinter::printMatrix(const basis_i &m)
{
  fprintf(f,"M :{%i %i : \n",m[0].size(),m.size());
  for(int i=0;i<m[0].size();i++)
    {
      for(int j=0;j<m.size();j++)
      {
	fprintf(f,"%i ",m[j][i]);
      }
      if(i==m[0].size()-1)fprintf(f,"}");
      fprintf(f,"\n");
    }
}


void AsciiPrinter::printTransposedMatrix(const basis_i &m)
{
}


void AsciiPrinter::printGroebnerBasis(const gbasis &g)
{
  fprintf(f,"(");
  for(gbasis::const_iterator i=g.begin();i!=g.end();i++)
    {
      if(i!=g.begin())fprintf(f,", ");
      printBinomial(*i);
    }
  fprintf(f,")");
}


void AsciiPrinter::printGroebnerBasisNewLine(const gbasis &g)
{
  fprintf(f,"(\n");
  for(gbasis::const_iterator i=g.begin();i!=g.end();i++)
    {
      if(i!=g.begin())fprintf(f,",\n");
      printBinomial(*i);
    }
  fprintf(f,"\n)\n");
}


void AsciiPrinter::printMonomialList(const gbasis &g)
{
  fprintf(f,"(");
  for(gbasis::const_iterator i=g.begin();i!=g.end();i++)
    {
      if(i!=g.begin())fprintf(f,", ");
      printMonomial(*i);
    }
  fprintf(f,")");
}


void AsciiPrinter::printMonomialListNewLine(const gbasis &g)
{
  fprintf(f,"(");
  for(gbasis::const_iterator i=g.begin();i!=g.end();i++)
    {
      if(i!=g.begin())fprintf(f,",\n");
      printMonomial(*i);
    }
  fprintf(f,"\n)\n");
}


void AsciiPrinter::printMonomialIdeal(const gbasis &g)
{
  fprintf(f,"(");
  for(gbasis::const_iterator i=g.begin();i!=g.end();i++)
    {
      if(i!=g.begin())fprintf(f,", ");
      printMonomial(*i,1);
    }
  fprintf(f,")");
}


void AsciiPrinter::printMonomialIdealList(const gfan &l)
{
  fprintf(f,"{ ");
  for(gfan::const_iterator i=l.begin();i!=l.end();i++)
    {
      if(i!=l.begin())fprintf(f,", ");
      printMonomialIdeal(*i);
    }
  fprintf(f,"} ");
}


void AsciiPrinter::printStandardPair(const StandardPair &g)
{
  fprintf(f,"( ");
  printMonomial(g.root);
  fprintf(f," , ");
  gbasis h;
  for(int i=0;i<g.face.size();i++)
    if(g.face[i])
      {
	vektor_i v(g.face.size());
	v[i]=1;
	h.push_back(v);
      }
  printMonomialIdeal(h);
  fprintf(f," )");
}


void AsciiPrinter::printStandardPairList(const StandardPairList &g)
{
  fprintf(f,"{");
  for(StandardPairList::const_iterator i=g.begin();i!=g.end();i++)
    {
      if(i!=g.begin())fprintf(f,", \n");
      printStandardPair(*i);
    }
  fprintf(f,"}\n");
}


void AsciiPrinter::printString(const string &s)
{
  fprintf(f,"%s",s.c_str());
}


//--------------------------------------------------
// XmlPrinter
//--------------------------------------------------
void XmlPrinter::printMonomial(const vektor_i &m, int sign)
{
}


void XmlPrinter::printBinomial(const vektor_i &b)
{
}


void XmlPrinter::printVector(const vektor_i &v)
{
}


void XmlPrinter::printVectorList(const gbasis &g)
{
}


void XmlPrinter::printMatrix(const basis_i &m)
{
  for(int i=0;i<m[0].size();i++)
    {
      fprintf(f,"<row>");
      for(int j=0;j<m.size();j++)
      {
	fprintf(f,"<entry>%i</entry>",m[j][i]);
      }
      fprintf(f,"</row>\n");
    }
}


void XmlPrinter::printTransposedMatrix(const basis_i &m)
{
}


void XmlPrinter::printGroebnerBasis(const gbasis &g)
{
}


void XmlPrinter::printGroebnerBasisNewLine(const gbasis &g)
{
}


void XmlPrinter::printMonomialList(const gbasis &g)
{
}


void XmlPrinter::printMonomialListNewLine(const gbasis &g)
{
}

void XmlPrinter::printMonomialIdeal(const gbasis &g)
{
}


void XmlPrinter::printMonomialIdealList(const gfan &l)
{
}


void XmlPrinter::printStandardPair(const StandardPair &g)
{
}


void XmlPrinter::printStandardPairList(const StandardPairList &g)
{
}


void XmlPrinter::printString(const string &s)
{
}
