#ifndef Printer_INCLUDED
#define Printer_INCLUDED

#include <string>
#include "vektor.h"
#include "flip.h" //only included because of gfan type
#include "decompose.h" //only included because of StandardPairList

// In time this file will replace print.h
//   ... and at some point the C-Style FILE * will also be removed

class Printer
{
 protected:
  FILE *f;
  virtual string variableIndexToString(int i);
 public:
  Printer(FILE *f){this->f=f;};
  virtual void printMonomial(const vektor_i &m, int sign=1)=0;
  virtual void printBinomial(const vektor_i &b)=0;
  virtual void printVector(const vektor_i &v)=0;
  virtual void printVectorList(const gbasis &g)=0;
  virtual void printMatrix(const basis_i &m)=0;
  virtual void printTransposedMatrix(const basis_i &m)=0;
  virtual void printGroebnerBasis(const gbasis &g)=0;
  virtual void printGroebnerBasisNewLine(const gbasis &g)=0;
  virtual void printMonomialList(const gbasis &g)=0;
  virtual void printMonomialListNewLine(const gbasis &g)=0;
  virtual void printMonomialIdeal(const gbasis &g)=0;
  virtual void printMonomialIdealList(const gfan &g)=0;
  virtual void printStandardPair(const StandardPair &g)=0;
  virtual void printStandardPairList(const StandardPairList &g)=0;
  virtual void printString(const string &s)=0;
};


class LatexPrinter:public Printer
{
  int mathModeLevel;
  void pushMathMode();
  void popMathMode();
 public:
  LatexPrinter(FILE *f):Printer(f){mathModeLevel=0;}
  virtual void printMonomial(const vektor_i &m, int sign=1);
  virtual void printBinomial(const vektor_i &b);
  virtual void printVector(const vektor_i &v);
  virtual void printVectorList(const gbasis &g);
  virtual void printMatrix(const basis_i &m);
  virtual void printTransposedMatrix(const basis_i &m);
  virtual void printGroebnerBasis(const gbasis &g);
  virtual void printGroebnerBasisNewLine(const gbasis &g);
  virtual void printMonomialList(const gbasis &g);
  virtual void printMonomialListNewLine(const gbasis &g);
  virtual void printMonomialIdeal(const gbasis &g);
  virtual void printMonomialIdealList(const gfan &g);
  virtual void printStandardPair(const StandardPair &g);
  virtual void printStandardPairList(const StandardPairList &g);
  virtual void printString(const string &s);
};


class AsciiPrinter:public Printer
{
 public:
  AsciiPrinter(FILE *f):Printer(f){}
  virtual char vectorLeftBrackets(){return '(';}
  virtual char vectorRightBrackets(){return ')';}
  virtual char vectorListLeftBrackets(){return '{';}
  virtual char vectorListRightBrackets(){return '}';}
  virtual void printMonomial(const vektor_i &m, int sign=1);
  virtual void printBinomial(const vektor_i &b);
  virtual void printVector(const vektor_i &v);
  virtual void printVectorList(const gbasis &g);
  virtual void printMatrix(const basis_i &m);
  virtual void printTransposedMatrix(const basis_i &m);
  virtual void printGroebnerBasis(const gbasis &g);
  virtual void printGroebnerBasisNewLine(const gbasis &g);
  virtual void printMonomialList(const gbasis &g);
  virtual void printMonomialListNewLine(const gbasis &g);
  virtual void printMonomialIdeal(const gbasis &g);
  virtual void printMonomialIdealList(const gfan &g);
  virtual void printStandardPair(const StandardPair &g);
  virtual void printStandardPairList(const StandardPairList &g);
  virtual void printString(const string &s);
};


class XmlPrinter:public Printer
{
 public:
  XmlPrinter(FILE *f):Printer(f){}
  virtual void printMonomial(const vektor_i &m, int sign=1);
  virtual void printBinomial(const vektor_i &b);
  virtual void printVector(const vektor_i &v);
  virtual void printVectorList(const gbasis &g);
  virtual void printMatrix(const basis_i &m);
  virtual void printTransposedMatrix(const basis_i &m);
  virtual void printGroebnerBasis(const gbasis &g);
  virtual void printGroebnerBasisNewLine(const gbasis &g);
  virtual void printMonomialList(const gbasis &g);
  virtual void printMonomialListNewLine(const gbasis &g);
  virtual void printMonomialIdeal(const gbasis &g);
  virtual void printMonomialIdealList(const gfan &g);
  virtual void printStandardPair(const StandardPair &g);
  virtual void printStandardPairList(const StandardPairList &g);
  virtual void printString(const string &s);
};


class TopcomPrinter:public AsciiPrinter
{
 public:
  TopcomPrinter(FILE *f):AsciiPrinter(f){}
  virtual char vectorLeftBrackets(){return '[';}
  virtual char vectorRightBrackets(){return ']';}
  virtual char vectorListLeftBrackets(){return '[';}
  virtual char vectorListRightBrackets(){return ']';}
};

#endif
