#include <stdio.h>

#include "toric.h"

static const bool uselll=1;
static const bool usemixedelim=0;

gbasis toric1(basis_i B, bool printComments)
{
  int kerdim;
  basis_i A=B;

  if(printComments)
    {
      printf("Computing a Gr\\\"obner basis for $I_A$ using algorithm 3.18\n\n");
      printf("Input matrix A:\n\n");
      printmatrix(B);
      printf("\n\n");
    }
  basis_i M=mlll(B);
  kerdim=0;
  while((kerdim<B.size())&&(B[kerdim].iszero()))kerdim++;

  //  toric1_kerdim=kerdim;

  if(printComments)
    {
      printf("Running the LLL algorithm on the column vectors of $A$ gives the identity:\n\n");
      printmatrixtransposed(B);
      printf("=");
      printmatrixtransposed(M);
      printf("$\\cdot$");
      printmatrixtransposed(A);
      printf("\n\n where the rows in the matrix to the right are the input and the rows in the matrix to the left are the output. The matrix in the middle describes their relation.\n\n");
      
      printf("$dim(ker(A))=%i$\n\n",kerdim);
    }

  basis_i M1(kerdim);

  for(int i=0;i<kerdim;i++)M1[i]=M[i];

  if(printComments)
    {
      printf("A lattice basis for the lattice kernel of $A$ is the rows of (coming from the relations between the input and output of the LLL algorithm):\n\n");
      printmatrixtransposed(M1);
    }
  if(uselll)
    {
      basis_i MM=mlll(M1);
		
      if(printComments)
	{
	  printf("\n\nRunning LLL once agian we find an LLL-reduced lattice basis for the lattice kernel of $A$. It is the rows of:\n\n");
	  printmatrixtransposed(M1);
	}
    }
  if(printComments)
    {
      printf("\n\nCorresponding to the pure binomials:\n\n");
      printt(M1);
    }

  gbasis MMM(kerdim);
  copy(M1.begin(),M1.end(),MMM.begin());

  if(printComments)
    {
      printf("The ideal generated by these binomials gets saturated with respect to each variable:\n\n");
    }

  for(int i=0;i<B.size();i++)
    {
      if(printComments)
	{
	  printf("Step %i:\n",i+1);
	}
      vektor_i::TERMORDER=REVERSELEXICOGRAPHIC;
      vektor_i::setcheapest(i);
      buchberger(MMM);
      gbasisminimal(&MMM);
      gbasisautoreduction(&MMM);
      if(printComments)
	{
	  printf("Current number of generators: %i\n\n",MMM.size());
	}
    }
  if(printComments)
    {
      printf("The resulting Gr\\\"obner basis for $I_A$ consists of %i elements:\n\n",MMM.size());
      printt(MMM);
    }

  return MMM;
}

gbasis toric_simple(basis_i B)
{
  int kerdim;

  int n;
  int d;

  printf("Computing a Gr\\\"obner basis for $I_A$ using algorithm 2.2:\n\n");
  
  printf("Input matrix A:\n\n");
  //  print(B);
  printmatrix(B);

  printf("\n\n");

  gbasis B2;

  d=B[0].size();
  n=B.size();

  for(int i=0;i<B.size();i++)
    {
      vektor_i v(B[0].size()+B.size());
      int j;
      for(j=0;j<B[0].size();j++)v[j]=-(B[i][j]);
      for(;j<v.size();j++)v[j]=(j-B[0].size()==i);

      B2.push_back(v);
    }
  fflush(stdout);

  printf("We form the ideal generated by (with indices shifted):\n\n");


  printt(B2);
  //print(B2);

  if(usemixedelim==0)
    {
      printf("Computing a Gr\\\"obner basis for the ideal with respect to the lexicographic term order...\n\n");

      vektor_i::TERMORDER=LEXICOGRAPHIC;
      vektor_i::setcheapest(0);
    }
  else
    {
      printf("Computing a Gr\\\"obner basis for the ideal with respect to a suitable elimination term order...\n\n");

      vektor_i::TERMORDER=MIXEDELIMINATION;
      vektor_i::mixedelimk=d;
    }

  buchberger(B2);
  printf("GBasis size:%i\n\n",B2.size());
  fflush(stdout);

  printf("Autoreducing...\n\n");
  gbasisminimal(&B2);
  gbasisautoreduction(&B2);
  printf("GBasis size:%i\n\n",B2.size());

  printf("Intersecting ideal with the (sub)polynomial ring\n\n");

  gbasis B3;

  for(gbasis::iterator i=B2.begin();i!=B2.end();i++)
    {
      int flag=1;
      for(int j=0;j<d;j++)if((*i)[j]!=0)flag=0;
      if(flag)
	{
	  vektor_i v(n);

	  for(int k=0;k<n;k++)
	    {
	      v[k]=(*i)[k+d];
	    }
	  B3.push_back(v);	  
	  //	  print(*i);
	} 
    }

  printf("The generators for the intersection (%i generators):\n\n",B3.size());
  printt(B3);

  if(usemixedelim==0)
    {
      printf("This is in fact a reduced Gr\\\"obner basis with respect to the lexicographic term order with $x_1\\succ x_2\\succ\\dots$(why?)\n\n");
    }
  else
    {
      printf("This is in fact a reduced Gr\\\"obner basis with respect to the graded reverse lexicographic term order with $x_1\\succ x_2\\succ\\dots$(why?)\n\n");
    }
  return B3;
}











