#ifndef VEKTOR_H__
#define VEKTOR_H__

#define LEXICOGRAPHIC 0
#define REVERSELEXICOGRAPHIC 1
#define MIXEDELIMINATION 2

#define MAXNUMBEROFVARIABLES 52

#include <stdio.h>
#include <cmath>
#include <iostream>
#include <vector>
#include <list>
using namespace std;

extern int numberofvektors;
extern int maxnumberofvektors;

#define UP(){numberofvektors++;if(numberofvektors>maxnumberofvektors)maxnumberofvektors=numberofvektors;}
#define DOWN(){numberofvektors--;}

template <class typ> class vektor{
public:	
  vector<typ> v;
  int support; 
public:
  vektor(const vektor &a):v(a.v),support(a.support){
    UP();
  }
  vektor(int n):v(n){
    support=0;
    UP();
    for(int i=0;i<n;i++)v[i]=typ(0.0);
  };
  vektor(const typ *p, int n):v(n){
    UP();
    support=0;for(int i=0;i<n;i++)v[i]=p[i];
  };
  
  ~vektor(){
    DOWN();
  };
  vektor(){
    support=0;
    UP();
  };
  
        void calcsupport(void)
	  {
	    support=0;
	    for(int i=0;i<v.size();i++)support=(support<<1)|(((v[i]>0)==true)&1);
	  }


	template<class T>
	  vektor(const vektor<T>& c):v(c.size()){
	  UP();
	  for(int i=0;i<size();i++)v[i]=typ(c[i]);}


	typ& operator[](int n){return (v[n]);}
	const typ& operator[](int n)const{return (v[n]);}
	
	unsigned int size()const{return v.size();};
	typ sum()const{typ f=0;for(int i=0;i<size();i++)f+=v[i];return f;};
	void resize(int n){v.resize(n,0);};
	void grow(int i){if(size()<i)resize(i);}
	inline friend vektor operator-(const vektor& q){return -1*q;};
	inline friend vektor operator*(typ s, const vektor& q){vektor p=q;for(int i=0;i<q.size();i++)p[i]*=s;return p;}
	inline friend vektor operator*(const vektor& p, const vektor& q){vektor p1=p;for(int i=0;i<p.size();i++)p1.v[i]*=q.v[i];return p1;}
	inline friend vektor operator+(const vektor& p, const vektor& q){vektor p1=p;for(int i=0;i<p.size();i++)p1[i]+=q[i];return p1;}
	inline friend vektor operator-(const vektor& p, const vektor& q){vektor p1=p;for(int i=0;i<p.size();i++)p1[i]-=q[i];return p1;}

	inline friend typ dot(const vektor& p, const vektor& q){typ s=0;for(int i=0;i<p.size();i++)s+=p[i]*q[i];return s;}

	vektor& operator+=(const vektor& q){for(int i=0;i<size();i++)v[i]+=q.v[i];return *this;}
	vektor& operator-=(const vektor& q){for(int i=0;i<size();i++)v[i]-=q.v[i];return *this;}

	bool operator==(const vektor & q)const{if(size()!=q.size())return false;for(int i=0;i<size();i++)if(v[i]!=q[i])return false;return true;}
	bool operator!=(const vektor & q)const {return !(operator==(q));} 

	static bool weightenable;
	static vektor<typ> weight;
	static bool weight_double_enable;
	static vektor<double> weight_double;
	static int TERMORDER;
	static int cheapest;

	static int cheaptable[MAXNUMBEROFVARIABLES];
	static int mixedelimk;
	static void setcheapest(int n)
	  {
	    for(int i=0;i<MAXNUMBEROFVARIABLES;i++)
	      {
		if(i<=n)cheaptable[i]=n-i*2;
		if(i>n)cheaptable[i]=0;
	      }
	    cheapest=n;
	  }
	static void setcheapest2(int n,int nvar)
	  {
	    assert(nvar<MAXNUMBEROFVARIABLES);
	    for(int i=0;i<nvar;i++)
	      {
		cheaptable[i]=((i+n)%nvar)-i;
	      }
	    cheapest=n;
	  }
	static void setCheapVector(list<int> const &l)
	  {
	    assert(l.size()<MAXNUMBEROFVARIABLES);
	    int i=0;
	    for(list<int>::const_iterator j=l.begin();j!=l.end();j++)
	      {
		cheaptable[i]=(*j)-i;
		i++;
	      }
	  }
	const int ordered() const
	  {
	    if(weightenable)
	      {
		if(weight_double_enable)
		  {
		    double s=0;
		    for(int i=0;i<v.size();i++)s+=v[i]*weight_double[i];
		    if(s>0)return 1;
		    if(s<0)return 0;
		  }
		else
		  {
		    if(dot(*this,weight)>0)return 1;
		    if(dot(*this,weight)<0)return 0;
		  }
 	      }
	    if(TERMORDER == LEXICOGRAPHIC)
	      {
		int n=v.size();
		for(int i=0;i<n;i++)
		  {
		    int index=cheaptable[i]+i;
		    if(v[index]<0)return 0;
		    if(v[index]>0)return 1;
		  }
		return 1;
	      }
	    else if(TERMORDER == REVERSELEXICOGRAPHIC)
	      {
		int n=v.size();
		for(int i=0;i<n;i++)
		  {
		    int index=cheaptable[i]+i;
		    if(v[index]<0)return 1;
		    if(v[index]>0)return 0;
		  }
		return 1;
	      }
	    else if(TERMORDER == MIXEDELIMINATION)
	      {
		int s=0;
		int k=mixedelimk;
		
		for(int i=0;i<k;i++)s+=v[i];
		if(s>0)return 1;
		if(s<0)return 0;
		for(int j=v.size()-1;j>=0;j--)
		  {
		    if(v[j]<0)return 1;
		    if(v[j]>0)return 0;
		  }
		return 1;
	      }
	  }
	bool operator<(const vektor &right)const
	  {
	    vektor v=*this -right;
	    if(v.iszero())return false;
	    if(v.ordered())return true;
	    return false;
	  }
	bool leadingTermLess(const vektor &right) const
	  {
	    return lt()<right.lt();  // should be inlined
	  }
	void order()
	  {
	    if(!ordered())*this=(typ)(-1.0)* *this;
	  }
	int divides(const vektor& q) const
	  {
	    int n=v.size();
	    for(int i=0;i<n;i++)
	      {
		if(v[i]>0)if(q.v[i]<v[i])return 0;
	      }
	    return 1;
	  }
	int dividesnonlt(const vektor& q) const
	  {
	    int n=v.size();
	    for(int i=0;i<n;i++)
	      {
		if(v[i]>0)if((-q.v[i])<v[i])return 0;
	      }
	    return 1;
	  }
	int reduce(const vektor& q)
	  {
	    if(q.divides(*this))
	      {
		*this=*this-q;
		return 1;
	      }
	    else return 0;
	  }
	inline friend vektor S(const vektor& p, const vektor& q)
	  {
	    return p-q;
	  }
	inline friend bool relativelyprime(const vektor& p, const vektor& q)
	  {
	    int n=p.size();
	    for(int t=0;t<n;t++)if((p[t]>0)&&(q[t]>0)) return false;
	    return true;
	  }
	int iszero() const
	  {
	    int n=v.size();
	    for(int i=0;i<n;i++)if(v[i]!=0)return 0;
	    return 1;
	  }
	inline vektor lt()const{vektor p=*this; for(int i=0;i<size();i++)if(p[i]<0)p[i]=0;return p;};
	inline vektor tl()const{vektor p=-*this; for(int i=0;i<size();i++)if(p[i]<0)p[i]=0;return p;};

	friend vektor multiply(vector<vektor> const &A, vektor const &v){vektor r(A[0].size());for(int i=0;i<A.size();i++)r+=v[i]*A[i];return r;}
};

typedef vektor<double> vektor_f;
typedef vektor<int> vektor_i;

typedef vector<vektor_f> basis_f;
typedef vector<vektor_i> basis_i;
typedef list<vektor_i> gbasis;

//basis_f tobasis_f(const char *s);
basis_f tobasis_f(const basis_i& i);
//basis_i tobasis_i(const char *s);
basis_i tobasis_i(const basis_f& i);

basis_i randommatrix_i(int rows, int cols, int max);

gbasis initialIdeal(const gbasis &g);

basis_i toIndexedMatrix(gbasis m);
gbasis toLinkedMatrix(basis_i m);
basis_i transposeMatrix(const basis_i &m);

void sortLex(gbasis *g);
bool areEqual(gbasis *g, gbasis *h);//sorts two vector sets without duplicates and compares them

bool isGraded(const gbasis &g);
int degree(const gbasis &monomialIdeal);

#endif




