#include "xml.h"
#include "normaliz.h"
#include "toric.h"
#include "printer.h"

class XmlTag
{
  char *name;
  FILE *f;
public:
  XmlTag(char *name, FILE *f)
  {
    this->name=name;
    this->f=f;
    fprintf(f,"<%s>",name);
  }
  ~XmlTag()
  {
    fprintf(f,"</%s>\n",name);
  }
};

static const char* boolToText(bool b)
{
  return b?"true":"false";
}

void writeExampleToXml(const basis_i &A, FILE *f)
{
  int n=A.size();
  int d=A[0].size();

  gbasis a=toLinkedMatrix(A);
  gbasis a2=normaliz(a);
  
  bool areEqual=false;
  if(a2.size()==a.size())
    {
     areEqual=true;
     gbasis::iterator j=a2.begin();
     for(gbasis::iterator i=a.begin();i!=a.end();i++)
       {
	 if(!(*i-*j).iszero()){areEqual=false;break;}
	 j++;
       }
    }

  gbasis g=toric1(A);

  vektor_i::setcheapest2(0,n);
  vektor_i::TERMORDER=REVERSELEXICOGRAPHIC;
  buchberger(g);
  gbasisminimal(&g);
  gbasisautoreduction(&g);

  XmlTag t("example",f);
  {
    XmlTag t("name",f);
    fprintf(f,"unknown");
  }
  {
    XmlTag t("matrix",f);
    XmlPrinter(f).printMatrix(A);
  }
  {
    XmlTag t("isGraded",f);
    fprintf(f,boolToText(isGraded(g)));
  }
  {
    XmlTag t("isNormal",f);
    fprintf(f,areEqual?"true":"maybe");
  }
  {
    XmlTag t("isDeltaNormal",f);
    fprintf(f,"maybe");
  }
  {
    XmlTag t("n",f);
    fprintf(f,"%i",n);
  }
  {
    XmlTag t("d",f);
    fprintf(f,"%i",d);
  }
  {
    XmlTag t("numberOfEdges",f);
  }
  {
    XmlTag t("numberOfVertices",f);
  }
  {
    XmlTag t("numberOfRegularTriangulations",f);
  }
}

